/* Monte-Carlo collision routines for particle species with background gas and
** each other.
** asigma1:  Ar + e -> Ar + e      scattering
** asigma2:  Ar + e -> Ar* + e     lumped excitation cross-section (inc 2 radiative and 2 metastable levels)
** asigma3:  Ar + e -> Ar+ + 2e     ionisation from ground state
** asigma4:  Ar+ + Ar -> Ar + Ar+   charge transfer (ions)
** asigma5:  Ar+ + Ar -> Ar+ + Ar   elastic scattering of ions
** asigma6:  Ar^m + e -> Ar+ + 2e   Ionization of excited states by electron impact
** asigma7:  Ar + e -> Ar^m + 2e   Excitation to metastable states by electron impact (subset of asigma2)
*/

#include "pdc1.h"
#include "xgrafix.h"

#define NEMAX    100
#define NIMAX    100

void newvel(SCALAR, SCALAR, SCALAR *, SCALAR *, SCALAR *, int);

/* Set factor for calculating electron mobility */
SCALAR mue_factor()
{
  return(1.25); /* anisotropic HBS cs set */
}

/*-------------------------------------------------------------*/
/* Monte Carlo scheme for electron and ion-neutrals collisions */

void argonmcc(int isp)
{
  static int init_flag=1, icorrect=0;
  static SCALAR ecol_extra, icol_extra, ecol_extra2;		
  static SCALAR max_sigmav_e, max_sigmav_i, max_sigmav_e2;	
  static SCALAR col_prob_e, col_prob_i, Cdt, N_ex_max=1;
  
  register int j, i;
  int N, k, nnp, s, index;
  SCALAR etov, dt_correct;
  SCALAR rand_num, v_sqrd, vel, sigma_total, sum_prob;
  SCALAR engy, rengy, frac, phi1;
  SCALAR cosphi, sinphi, coschi, sinchi;
  SCALAR vneutr, vneutth, vneutz;
  SCALAR temp;						

  if (init_flag)
  {
    /*---------------------------------------*/
    /* Calculating the null collision prob.  */
    
    if (ionspecies) 
    {
       ionsp = 1;
       vg_therm = sqrt(T_gas/(Escale[ionsp]));
    }
    if(ecollisional)
    {
      ecolsp = 0;
      max_sigmav_e = 0.0;
      etov = 2.0*ECHARGE/mass[ecolsp];
      for (i=0; i< 10*NEMAX; i++)
      {
	engy = 0.1*i;
	max_sigmav_e= max(max_sigmav_e, sqrt(engy*etov)*(asigma1(engy)+
                      asigma2(engy)+asigma3(engy)));
      }
      col_prob_e = 1.0 -exp(-max_sigmav_e*gas_den*sp_k[ecolsp]*dt);

  #ifdef MPI_1D
      if (my_rank == ROOT){
        printf("Gas density = %lf\n", gas_den);
        printf("Electron coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", 
            col_prob_e, max_sigmav_e, max_sigmav_e*gas_den*dt*sp_k[ecolsp]);
      }
  #else                      
      printf("Gas density = %lf\n", gas_den);
      printf("Electron coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", 
            col_prob_e, max_sigmav_e, max_sigmav_e*gas_den*dt*sp_k[ecolsp]);
  #endif

      max_sigmav_e2 = 0.0;		
/* If radiation transport turned on calculate ionisation of metastables by
electron impact */
      if (RT_flag) 
      {
        for (i=0; i<= 2000; i++) 
        {
          engy = 0.1*i;
          max_sigmav_e2= max(max_sigmav_e2, sqrt(engy*etov)*asigma6(engy));
        }
        Cdt=max_sigmav_e2*dt*sp_k[ecolsp];
      }
    }
    
    if(icollisional)
    {
      icolsp = 1;
      max_sigmav_i = 0.0;
      etov = 2.0*ECHARGE/mass[icolsp];
      for (i=0; i<10*NIMAX; i++)
      {
	engy = 0.1*i;
	max_sigmav_i= max(max_sigmav_i, sqrt(engy*etov)
			  *(asigma4(engy)+asigma5(engy)));
      }
      col_prob_i = 1 -exp(-max_sigmav_i*gas_den*sp_k[icolsp]*dt);

  #ifdef MPI_1D
      if (my_rank == ROOT) /* MPI modification; only ROOT prints */
        printf("Ar+ coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", col_prob_i, max_sigmav_i, max_sigmav_i*gas_den*dt*sp_k[icolsp]);
  #else
      printf("Ar+ coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", col_prob_i, max_sigmav_i, max_sigmav_i*gas_den*dt*sp_k[icolsp]);
  #endif

    }
    ecol_extra = icol_extra = ecol_extra2 = 0.5;
    init_flag = 0;
  }
  
  /*------------------------------------------------------------* 
       Electron Collision : Ar* + e  -> Ar+ + 2e  ( by LEEHJ )
   *------------------------------------------------------------*/
  if(RT_flag && ecollisional && isp==ecolsp) 
  {
    dt_correct=gas_den/(N_ex_max*10);
    icorrect++;
    for (N_ex_max=0,j=0;j<nc_RT;j++) N_ex_max=max(N_ex_max, sp_ex[j]+sp_exm[j]);
    if (dt_correct<=icorrect) 
    {
      if(theRunWithXFlag) for (j=0; j<ng; j++) mccrate[5][j] = 0;
      ecol_extra2 += np[ecolsp]* (1-exp(-N_ex_max*icorrect*Cdt));
      N=ecol_extra2;
      ecol_extra2-=N;
  
      nnp = np[ecolsp];
      for(j=0; j< N; j++)
      {
        nnp--;
        index= nnp*frand();
  
        swap(r[ecolsp],index,nnp);
        swap(v_r[ecolsp],index,nnp);
        swap(v_theta[ecolsp],index,nnp);
        swap(v_z[ecolsp],index,nnp);
  
        v_sqrd = v_r[ecolsp][nnp]*v_r[ecolsp][nnp] 
               + v_theta[ecolsp][nnp]*v_theta[ecolsp][nnp]  
               + v_z[ecolsp][nnp]*v_z[ecolsp][nnp];
        engy= Escale[ecolsp]*v_sqrd;
        vel = sqrt(v_sqrd);
        sigma_total = max_sigmav_e2/(vel*vscale);
        rand_num= frand();     

        s=j_pointer(ecolsp,nnp)/nc_ratio; 
        temp=(sp_ex[s]+sp_exm[s])/N_ex_max;
        if (engy>e_miz && rand_num <= (sum_prob=asigma6(engy)/sigma_total)*temp)
        {
        /* first normalize the velocity components by the speed, vel */
          v_r[ecolsp][nnp] /= vel;
          v_theta[ecolsp][nnp] /= vel;
          v_z[ecolsp][nnp] /= vel;

          engy -= e_miz;
          rengy = 2.0*tan(frand()*atan(engy/20.0));
          engy -= rengy;
      
          /*-----------------------------------*/
          /* scatter the created electron	    */

          vel = sqrt(rengy/Escale[ecolsp]);
          k = np[ecolsp];
          v_r[ecolsp][k] = v_r[ecolsp][nnp];
          v_theta[ecolsp][k] = v_theta[ecolsp][nnp];
          v_z[ecolsp][k] = v_z[ecolsp][nnp];
          newvel(rengy,vel,&v_r[ecolsp][k],&v_theta[ecolsp][k],&v_z[ecolsp][k],0);
          r[ecolsp][k] = r[ecolsp][nnp];

          /* determine the charge density due to new electron */
        
          frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
          sp_n_mcc[ecolsp][s]  += frac;
          sp_n_mcc[ecolsp][s+1]+= (1.0 - frac); 

          /*-------------------------------------------------------*/
          /* create ion and assign velocities if ion species exist */

          if (ionspecies)
          {
            k = np[ionsp];
            maxwellv(&v_r[ionsp][k],&v_theta[ionsp][k],&v_z[ionsp][k],vg_therm);
            r[ionsp][k] = r[ecolsp][nnp];
  
            /* determine the charge density due to newly created ion */
            sp_n_mcc[ionsp][s]  += frac;
            sp_n_mcc[ionsp][s+1]+= (1.0 -  frac); 
          }

          /*---------------------------------------*/
          /* finally scatter the incident electron */

          vel = sqrt(engy/Escale[ecolsp]);
          newvel(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], 
                 &v_z[ecolsp][nnp], 0);

          if(++np[ionsp] >maxnp[ionsp] || ++np[ecolsp] >maxnp[ecolsp])
          {	  
          #ifdef MPI_1D
	          if (my_rank == ROOT)
               printf("mcc(Ionization): too many ions created ");
	       #else
             printf("mcc(Ionization): too many ions created ");
	       #endif
             exit(1);
          }

          if(theRunWithXFlag)
          {
            mccrate[5][s]  += frac/vol[s]/icorrect;
            mccrate[5][s+1]+= (1.0 - frac)/vol[s+1]/icorrect; 
          }

          /*---------------------------------------*/
          /*      Reduce excited state density     */
          k=s/nc_ratio;
          frac=sp_ex[k]/(sp_ex[k]+sp_exm[k]);
          sp_ex[k]-=frac/vol[s];
          sp_exm[k]-=(1-frac)/vol[s];

        }   /* end of test for type of collision */
      }    /* end of for(j=0; j< N; j++)         */
      icorrect=0;
    }     /* end of if (dt_correct<=icorrect)    */
  }  
  /*------------------------------------------------------------* 
             Electron Collision : Ar  + e  
   *------------------------------------------------------------*/
  if(ecollisional && isp==ecolsp)
  {
    if(theRunWithXFlag) {
      for (j=0; j<ng; j++) {
        mccrate[0][j] = mccrate[1][j] = mccrate[2][j] = mccrate[6][j] =
        Pel[j] = Pie[j] = 0.0;
      }
     
      nu_ex = nu_el = nu_iz = 0;
   #ifdef POW
      els_loss = 0.0;
   #endif
    }

 #ifdef POW
    else {
      els_loss = 0.0;
      nu_ex = nu_iz = 0;
    }
 #endif

    ecol_extra += np[ecolsp]*col_prob_e;
    N = ecol_extra;
    ecol_extra -= N;
    
    nnp = np[ecolsp];

    for(j=0; j< N; j++)
    {
      nnp--;
      index= nnp*frand();

/* Move colliding particle to end of array, so it isn't chosen twice */
      swap(r[ecolsp],index,nnp);
      swap(v_r[ecolsp],index,nnp);
      swap(v_theta[ecolsp],index,nnp);
      swap(v_z[ecolsp],index,nnp); 

   /*------------------------------------------------------------*/
      /* determine if a collision occurs */
      v_sqrd = v_r[ecolsp][nnp]*v_r[ecolsp][nnp] +v_theta[ecolsp][nnp]*v_theta[ecolsp][nnp]
	     +v_z[ecolsp][nnp]*v_z[ecolsp][nnp];
      engy= Escale[ecolsp]*v_sqrd;
      vel = sqrt(v_sqrd);
      sigma_total = max_sigmav_e/(vel*vscale);
      rand_num= frand();
      
      /*------------------------------------------------------------*/
      /* determine the type of collision and calculate new velocities, etc */
      /*------------------------------------------------------------*/
      /* if the collision is elastic */
      
      if (rand_num <= (sum_prob=asigma1(engy)/sigma_total) )
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
	/* scatter the electron */
	newvel(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 1);

        /* collect diagnostics*/
        if(theRunWithXFlag)
        {
          s = j_pointer(ecolsp,nnp);
	  frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
	  temp = frac/vol[s];
	  mccrate[0][s]  += temp;
	  Pel[s] += temp*en_loss;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[0][s+1]+= temp;
	  Pel[s+1] += temp*en_loss;
          nu_el ++; 
       #ifdef POW
          els_loss += en_loss;
       #endif
        }
     #ifdef POW
        else els_loss += en_loss;
     #endif
      }
      /**********************************/
      /* Excitation collision           */ 
      /* First check for metastable collisions using asigma7 (which is a subset of the TOTAL excitation x-s asigma2). 
       Excitation to the radiative state (effective x-sect = asigma2 - asigm7) will be tested in the next section. 
       nb sum_prob is NOT increased here, but in next section, where the total excitation probability is used. This 
       format requires metastable excitation to be tested before radiative excitation */

      else if (engy>e_me && rand_num<=(sum_prob + asigma7(engy)/sigma_total))
      {
        /* first normalize the velocity components by the speed, vel */
        v_r[ecolsp][nnp] /= vel;
        v_theta[ecolsp][nnp] /= vel;
        v_z[ecolsp][nnp] /= vel;

        /* subtract the excitation energy */
        engy -= e_me;
        vel   = sqrt(fabs(engy)/Escale[ecolsp]);

        /* scatter the electron */
        newvel(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 0);

        /* Determine the excitation profile */
        if(theRunWithXFlag)
        {
          s = j_pointer(ecolsp,nnp);
          frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
	  temp = frac/vol[s];
          mccrate[6][s]  += temp;
	  Pie[s]  += e_me*temp;
	  temp = (1.0 - frac)/vol[s+1]; 
          mccrate[6][s+1]+= temp;
	  Pie[s+1]+= e_me*temp; 
          nu_ex ++; 
        }
     #ifdef POW
        else nu_ex++;
     #endif 
      }

      /*------------------------------------------------------------*/
      /* Now check for radiative excitation and increase sum_prob by total
         excitation probability */
      
      else if (engy > eexc && rand_num <= (sum_prob += asigma2(engy)/sigma_total))
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
	/* subtract the excitation energy */
	engy -= eexc;
	vel   = sqrt(fabs(engy)/Escale[ecolsp]);
	
	/* scatter the electron */
	newvel(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 0);
	
	/* Determine the excitation profile */
	if(theRunWithXFlag)
	{
          s = j_pointer(ecolsp,nnp);
	  frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
	  temp = frac/vol[s];
	  mccrate[1][s]  += temp;
	  Pie[s]  += eexc*temp;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[1][s+1]+= temp; 
	  Pie[s+1]+= eexc*temp; 
          nu_ex ++; 
	}
     #ifdef POW
        else nu_ex++;
     #endif 
      }  

      /*------------------------------------------------------------*/
      /* if the collision is ionization, add the released electron and ion first */
      
      else if(engy>eion && rand_num <= (sum_prob += asigma3(engy)/sigma_total))
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
        /*------------------------------------------------------------*/
	/* subtract the ionisn energy and partition the remaining energy */
	
	engy -= eion;
	rengy = 10.0*tan(frand()*atan(engy/20.0));
	engy -= rengy;
	
        /*------------------------------------------------------------*/
	/* scatter the created electron */
	
	vel = sqrt(rengy/Escale[ecolsp]);
	k = np[ecolsp];
	v_r[ecolsp][k] = v_r[ecolsp][nnp];
	v_theta[ecolsp][k] = v_theta[ecolsp][nnp];
	v_z[ecolsp][k] = v_z[ecolsp][nnp];
	newvel(rengy, vel, &v_r[ecolsp][k], &v_theta[ecolsp][k], &v_z[ecolsp][k], 0);
	r[ecolsp][k] = r[ecolsp][nnp];

        /* determine the charge density due to new electron */
        s = j_pointer(ecolsp,nnp);
	frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
        sp_n_mcc[ecolsp][s]  += frac;
        sp_n_mcc[ecolsp][s+1]+= (1.0 -  frac); 
	
        /*------------------------------------------------------------*/
	/* create ion and assign velocities if ion species exist */
	
        if (ionspecies)
        {
	  k = np[ionsp];
	  maxwellv(&v_r[ionsp][k], &v_theta[ionsp][k], &v_z[ionsp][k], vg_therm);
	  r[ionsp][k] = r[ecolsp][nnp];

          /* determine the charge density due to newly created ion */
          sp_n_mcc[ionsp][s]  += frac;
          sp_n_mcc[ionsp][s+1]+= (1.0 -  frac); 

        }

        /*------------------------------------------------------------*/
	/* finally scatter the incident electron */
	
	vel = sqrt(engy/Escale[ecolsp]);
	newvel(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 0);
	
	if(++np[ionsp] >maxnp[ionsp] || ++np[ecolsp] >maxnp[ecolsp])
	{  
   #ifdef MPI_1D
      if (my_rank == ROOT) printf("mcc(Ionization): too many ions created ");
   #else
      printf("mcc(Ionization): too many ions created ");
   #endif

	 exit(1);
	}
	
        /*------------------------------------------------------------*/
	/* Determine the ionisation profile */
	if(theRunWithXFlag)
	{
	  temp = frac/vol[s];
	  mccrate[2][s]  += temp;
	  Pie[s]  += eion*temp;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[2][s+1]+= temp; 
	  Pie[s+1]+= eion*temp; 
          nu_iz ++;
	 }
      #ifdef POW
         else nu_iz++;
      #endif
      }   /* end of test for type of collision */
    }     /* end of loop over number colliding electrons*/
  }       /* end of "if (ecollisional)" */

  /*------------------------------------------------------------*/

  if(icollisional && isp==icolsp)
  {
    if(theRunWithXFlag)
      for (j=0; j<ng; j++) mccrate[3][j] = mccrate[4][j] = Pcx[j] = 0.0;

 #ifdef POW
    cx_loss = 0.0;
 #endif
      
    icol_extra += np[icolsp]*col_prob_i;
    N = icol_extra;
    icol_extra -= N;
    nnp = np[icolsp];

   /*------------------------------------------------------------*/
    for(j=0; j< N; j++)
    {
      nnp--;
      index= nnp*frand();     /* Pick a random ion to collide */
      
/* Move colliding particle to end of array, so it isn't chosen twice */
      swap(r[icolsp],index,nnp);
      swap(v_r[icolsp],index,nnp);
      swap(v_theta[icolsp],index,nnp);
      swap(v_z[icolsp],index,nnp); 

   /*------------------------------------------------------------*/
   /* Check for type of collision */

      maxwellv(&vneutr, &vneutth, &vneutz, vg_therm);
      /* Determine relative velocity btwn neutral and ion */
      v_r[icolsp][nnp] -= vneutr;
      v_theta[icolsp][nnp] -= vneutth;
      v_z[icolsp][nnp] -= vneutz;
      v_sqrd = (v_r[icolsp][nnp]*v_r[icolsp][nnp]
          +v_theta[icolsp][nnp]*v_theta[icolsp][nnp] +v_z[icolsp][nnp]*v_z[icolsp][nnp]);
      engy= Escale[icolsp]*v_sqrd;
      vel = sqrt(v_sqrd);
      sigma_total = max_sigmav_i/(vel*vscale);
      rand_num= frand();
      
       /*------------------------------------------------------------*/
      /* if the collision is scattering */
      
      if (rand_num <= (sum_prob = asigma5(engy)/sigma_total))
      {
	SCALAR up1, up2, up3, mag;
	SCALAR r11, r12, r13, r21, r22, r23, r31, r32, r33;
	
	coschi= sqrt(frand());
	sinchi= sqrt(1. -coschi*coschi);
	
	phi1  = TWOPI*frand();
	cosphi= cos(phi1);
	sinphi= sin(phi1);
	
	r13 = v_r[icolsp][nnp]/vel;
	r23 = v_theta[icolsp][nnp]/vel;
	r33 = v_z[icolsp][nnp]/vel;
	
	if(r33 == 1.0) { up1= 0;  up2= 1;  up3= 0; }
	else           { up1= 0;  up2= 0;  up3= 1; }
	
	r12 = r23*up3 -r33*up2;
	r22 = r33*up1 -r13*up3;
	r32 = r13*up2 -r23*up1;
	mag = sqrt(r12*r12 + r22*r22 + r32*r32);
	
	r12/= mag;
	r22/= mag;
	r32/= mag;
	
	r11 = r22*r33 -r32*r23;
	r21 = r32*r13 -r12*r33;
	r31 = r12*r23 -r22*r13;
	
	v_r[icolsp][nnp]= vel*coschi*(r11*sinchi*cosphi 
              +r12*sinchi*sinphi +r13*coschi);
	v_theta[icolsp][nnp]= vel*coschi*(r21*sinchi*cosphi 
              +r22*sinchi*sinphi +r23*coschi); 
	v_z[icolsp][nnp]= vel*coschi*(r31*sinchi*cosphi 
              +r32*sinchi*sinphi +r33*coschi);

     /* Count number of elastic collisions */ 
        if(theRunWithXFlag)
	{
          s = j_pointer(icolsp,nnp);
	  frac= (r_grid[s+1] - r[icolsp][nnp])/dr_n[s];
	  mccrate[4][s]  += frac/vol[s]/sp_k[icolsp];
	  mccrate[4][s+1]+= (1.0 - frac)/vol[s+1]/sp_k[icolsp]; 
	}
      }   /* end of "if rand_num < sigma " loop */
      
       /*------------------------------------------------------------*/
      /* if the collision is charge exchange */
      
      else if (rand_num <= (sum_prob += asigma4(engy)/sigma_total))
      {
	v_r[icolsp][nnp] = v_theta[icolsp][nnp] = v_z[icolsp][nnp] = 0.0;
	
     /* Count number of charge exchange collisions */ 
        if(theRunWithXFlag)
	{
          s = j_pointer(icolsp,nnp);
	  frac= (r_grid[s+1] - r[icolsp][nnp])/dr_n[s];
	  temp = frac/vol[s]/sp_k[icolsp];
	  mccrate[3][s]  += temp;
	  Pcx[s] += engy*temp;
	  temp = (1.0 - frac)/vol[s+1]/sp_k[icolsp];
	  mccrate[3][s+1]+= temp; 
	  Pcx[s+1] += engy*temp;
       #ifdef POW
          cx_loss += engy;
       #endif
	}
     #ifdef POW
        else cx_loss += engy;
     #endif
      }  /* end of "else rand_num < sigma + dsig" loop */

    /* Add neutral velocity back to ion velocity */

      v_r[icolsp][nnp] += vneutr;
      v_theta[icolsp][nnp] += vneutth;
      v_z[icolsp][nnp] += vneutz;
    }    /* end of loop of colliding particles */
  }      /* end of if icollisional loop */

}       /* end of mcc subroutine */

/*************************************************************
**  Calculates new velocities for particles which have 
**  just made a collision
**************************************************************/
void newvel(SCALAR energy, SCALAR vel, SCALAR *vr, SCALAR *vth, SCALAR *vz, int e_flag)
{
  SCALAR phi1, cosphi, sinphi, coschi, sinchi, up1, up2, up3;
  SCALAR mag, r11, r12, r13, r21, r22, r23, r31, r32, r33;
  SCALAR tmp;

  if(energy < 1e-30)  coschi = 1.0;
  else  coschi = (energy +2.0 -2.0*pow(energy +1.0,frand()))/energy;
  sinchi= sqrt(1. - coschi*coschi);
  
  phi1  = TWOPI*frand();
  cosphi= cos(phi1);
  sinphi= sin(phi1);
  
  if (e_flag) {
     tmp = 2.0*mass[ecolsp]*(1-coschi)/mass[ionsp];
     vel *= sqrt(1.0 - tmp);
     en_loss = energy*tmp;
  }
  
  r13 = *vr;
  r23 = *vth;
  r33 = *vz;
  
  if(r33 == 1.0) { up1= 0;  up2= 1;  up3= 0; }
  else           { up1= 0;  up2= 0;  up3= 1; }
  
  r12 = r23*up3 -r33*up2;
  r22 = r33*up1 -r13*up3;
  r32 = r13*up2 -r23*up1;
  mag = sqrt(r12*r12 + r22*r22 + r32*r32);
  
  r12/= mag;
  r22/= mag;
  r32/= mag;
  
  r11 = r22*r33 -r32*r23;
  r21 = r32*r13 -r12*r33;
  r31 = r12*r23 -r22*r13;
  
  *vr= vel*(r11*sinchi*cosphi +r12*sinchi*sinphi +r13*coschi);
  *vth= vel*(r21*sinchi*cosphi +r22*sinchi*sinphi +r23*coschi); 
  *vz= vel*(r31*sinchi*cosphi +r32*sinchi*sinphi +r33*coschi);
}

/*---------------------------------------------------------------------------*/
/* XSections for Argon-Electron Collisions                    
Curve fittings from Helen Smith's thesis                   
Experimental data used for fitting:

Electron-Argon Neutral 
----------------------
Excitation: 
     deHeer, Jansen and Kaay, J.Phys.B, 12, 979 (1979)
Ionisation: 
     Krishnakumar & Srivastava J.Phys.B. 21, 1055 (1988)
     Vikor et al, FIZIKA 21, 345 (1989)
Elastic 0 - 20 : 
     Ferch, Granitza, Masche & Raith J.Phys.B, 18, 967 (1985)
       20 - 3000 :
     deHeer, Jansen and Kaay, J.Phys.B, 12, 979 (1979)

Ar+ - Ar
----------------------
Elastic Scattering: 
   McDaniel "Collision Phenomena in Ionised Gases" (1964)
Charge Exchange:
   McDaniel "Collision Phenomena in Ionised Gases" (1964)
   Rapp & Francis, J.Chem. Phys 37, 2631 (1962)
   Hegerberg, Stefansson & Elford, J.Phys.B 11, 133 (1978)    */

/*------------------------------------------------------------*/
/*  Total e- Scattering Cross-section                            */
/*-------------------------------------------------------------*/
SCALAR asigmaT(SCALAR energy)
{
  return(asigma1(energy) + asigma2(energy) + asigma3(energy));
}

/*------------------------------------------------------------*/
/*  e + Ar -> e + Ar  Elastic                                  */
/*-------------------------------------------------------------*/
SCALAR asigma1(SCALAR energy)
{
  int i;
  SCALAR els_sigma, alpha;
  static SCALAR *elastic1, *elastic2;
  static int init_flag=1;

  /******  Initialization   *********/
  if(init_flag) {
    SCALAR engy;
    SCALAR amp_1 = 8.32414e-22, pwr_1 = -1.14818;
    SCALAR amp_2 = 1.10478e-20, pwr_2 = 1.25583;
    SCALAR amp_3 = 1.17039e-18, pwr_3= -0.669049;

    elastic1 = (SCALAR *) malloc(101*sizeof(SCALAR));
    elastic2 = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

    /*****  With the Ramseur min.  *****/
    elastic1[0] = amp_1*pow(0.005,pwr_1);
    for(i=1; i<101; i++) {
      engy = .01*i;
      if(engy < 0.345) elastic1[i]= amp_1*pow(engy,pwr_1);
      else elastic1[i]= amp_2*pow(engy,pwr_2);
    }

    for(i=1; i<NEMAX; i++) {
      engy = i;
      if (engy < 12.0)
        elastic2[i]= amp_2*pow(engy,pwr_2);
      else elastic2[i] = amp_3*pow(engy,pwr_3);
    }
    init_flag =0;
  }
  /*--------------------------------------------------------------------*/

  if(energy < 1.0) {
    i= 100*energy;
    alpha= 100*energy -i;
    els_sigma = elastic1[i] + alpha*(elastic1[i+1] - elastic1[i]);
  }
  else {
    i= energy;
    if(i < NEMAX-1) {
      alpha= energy -i;
      els_sigma = elastic2[i] + alpha*(elastic2[i+1] - elastic2[i]);
    }
    else
      els_sigma = elastic2[NEMAX-1];
  }

  return(els_sigma);
}
/*------------------------------------------------------------------------*/
/* Momentum transfer cross-section - convert from elastic x-s assuming     
anisotropic distribution used by V.Vahedi et al                           */
/*------------------------------------------------------------------------*/
SCALAR asigma1a(SCALAR energy)
{
  SCALAR temp, convert, q_mtm;
  temp = log(1 + energy);
  convert = 2.0*(energy - temp)/(energy*temp);
  q_mtm = convert*asigma1(energy);
  return(q_mtm);
}
/*---------------------------------------------------------------------------*/
/*  e + Ar -> e + Ar  Momentum Transfer elastic scattering */
/* Data taken from 
       M. Suzuki, T.Taniguchi, H.Tagashira, J.Phys.D.: Appl. Phys. 23, 842 (1990)
*/
/*---------------------------------------------------------------------------*/
SCALAR asigma1b(SCALAR energy)
{
  int i;
  static int init_flag=1;
  SCALAR mtm_sigma, alpha;
  static SCALAR *mtm_1, *mtm_2;

  /******  Initialization   *********/
  if(init_flag)
  {
    int N=22, i, j=1;
    SCALAR engy;
    SCALAR energy[23] = { 0.0, 0.05, 0.11, 0.14, 0.2, 0.25, 0.32, 0.4,
         0.5, 0.65, 0.8, 1.0, 1.5, 2.0, 3.0, 6.0, 8.0, 10.0, 12.0,
         15.0, 20.0, 26.0, 30.0};

    SCALAR sigma[23] = { 8.0, 1.84, 0.342, 0.235, 0.177, 0.156, 0.151,
         0.182, 0.283, 0.47, 0.68, 1.05, 1.74, 2.48, 4.07, 8.7, 11.7, 13.8,
         14.5, 13.2, 10.4, 8.3, 7.2};

    for (i=0; i<N+1; i++) sigma[i] *= 1.0e-20; /* convert x-sect from angstroms to m^2 */

    mtm_1 = (SCALAR *) malloc(101*sizeof(SCALAR));
    mtm_2 = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

    for(i=0; i<101; i++)
    {
        engy = 0.01*i;
        while (engy > energy[j] && j<N) j++;
        while (engy < energy[j-1]) j--;
        mtm_1[i]=sigma[j]+(sigma[j]-sigma[j-1])*(engy-energy[j])/(energy[j]-energy[j-1]);
    }
    for(i=1; i<NEMAX; i++)
    {
       engy = i;
       while (engy > energy[j] && j<N) j++;
       while (engy < energy[j-1]) j--;
       mtm_2[i]=sigma[j]+(sigma[j]-sigma[j-1])*(engy-energy[j])/(energy[j]-energy[j-1]);
       if (mtm_2[i] < 0) mtm_2[i] = 0;
    }
    init_flag =0;
  }
  /*--------------------------------------------------------------------*/
  if(energy < 1.0)
  { 
    i= 100*energy;
    alpha= 100*energy -i;
    mtm_sigma = mtm_1[i] + alpha*(mtm_1[i+1] - mtm_1[i]);
  }
  else
  { 
    i= energy;
    if(i < NEMAX-1)
    {
      alpha= energy -i;
      mtm_sigma = mtm_2[i] + alpha*(mtm_2[i+1] - mtm_2[i]);
    }
    else
      mtm_sigma = mtm_2[NEMAX-1];
  }
  
  return(mtm_sigma);
}
/*---------------------------------------------------------------------------*/
/*  e + Ar -> e + Ar  Excitation    */
/*---------------------------------------------------------------------------*/
SCALAR asigma2(SCALAR energy)
{
  int i;
  SCALAR exc_sigma, alpha;
  static int init_flag=1;
  static SCALAR *excit;

  /*-----  Initialization   --------*/
  if(init_flag)
  {
      SCALAR engy, x;
      SCALAR a = 0.18, b = 0.85, sig_exc0=1.1e-17;

      excit = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

      for(i=0; i<NEMAX; i++)
      {
          engy = i;
          x = engy/eexc;
          if(x < 1.0)
              excit[i] = 0;
          else
             excit[i] = sig_exc0/(pow(eexc,2.0)*pow(x,b))*(a*x-a)/(a*x+1);
      }
      init_flag =0;
  }
  /****************************/

  i= energy;
  if(i < NEMAX-1)
  {
    alpha= energy -i;
    exc_sigma = excit[i] + alpha*(excit[i+1] - excit[i]);
  }
  else
    exc_sigma = excit[NEMAX-1];

  return(exc_sigma);
}

/*---------------------------------------------------------------------------*/
/*  e + Ar -> e + e + Ar+  Ion.     */
/*---------------------------------------------------------------------------*/

SCALAR asigma3(SCALAR energy)
{
  int i;
  SCALAR ion_sigma, alpha;
  static int init_flag=1;
  static SCALAR *ioniz;

  /******  Initialization   *********/
  if(init_flag)
  {
    SCALAR engy, x;
    SCALAR a = 0.5, b = 0.87, c=1.4, sig_ion0=6.5e-17;
    ioniz = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

    for(i=0; i<NEMAX; i++)
    {
      engy = i;
      x = engy/eion;
      if(x < 1.0) ioniz[i] = 0;
      else
        ioniz[i] = sig_ion0/(eion*eion)/pow(x,b)*pow((a*x - a)/(a*x + 1),c);
    }
    init_flag =0;
  }
  /*--------------------------------------------------------------------*/

  i= energy;
  if(i < NEMAX-1)
  {
    alpha= energy -i;
    ion_sigma = ioniz[i] + alpha*(ioniz[i+1] - ioniz[i]);
  }
  else
    ion_sigma = ioniz[NEMAX-1];

  return(ion_sigma);
}
/*---------------------------------------------------------------------------*/
/* Argon Ion-neutral Collision Cross-sections
*/

/*---------------------------------------------------------------------------*/
/*  Ar + Ar+ -> Ar+ + Ar  Charge Exchange
*/
/*---------------------------------------------------------------------------*/

SCALAR asigma4(SCALAR energy)
{
  int i;
  SCALAR cx_sigma, alpha;
  static int init_flag=1;
  static SCALAR *cx1, *cx2;

  /******  Initialization   *********/
  if(init_flag)
  {
    SCALAR engy;

    cx1 = (SCALAR *) malloc(1001*sizeof(SCALAR));
    cx2 = (SCALAR *) malloc(NIMAX*sizeof(SCALAR));
    cx1[0] = 11.02e-19; /* cx[en = 1e-4] */
    cx2[0] = 11.02e-19;

    for(i=1; i<1001; i++) {
      engy = 0.001*i;
      cx1[i] = 1.0e-20*(7.0 - 0.38*log(engy))*(7.0 - 0.38*log(engy));
    }

    for(i=1; i<NIMAX; i++)
    {
      engy = i;
      cx2[i] = 1.0e-20*(7.0 - 0.38*log(engy))*(7.0 - 0.38*log(engy));
    }
    init_flag =0;
  }
  /*--------------------------------------------------------------------*/

  if(energy < 1.0) {
    i= 1000*energy;
    alpha= 1000*energy -i;
    cx_sigma = cx1[i] + alpha*(cx1[i+1] - cx1[i]);
  }
  else {
    i= energy;
    if(i < NIMAX-1)
    {
      alpha= energy -i;
      cx_sigma = cx2[i] + alpha*(cx2[i+1] - cx2[i]);
    }
    else
      cx_sigma = cx2[NIMAX-1];
  }

  return(cx_sigma);
}

/*---------------------------------------------------------------------------*/
/*  Ar + Ar+ -> Ar + Ar+   Elastic  Scattering
*/
/*---------------------------------------------------------------------------*/

SCALAR asigma5(SCALAR energy)
{
  int i;
  SCALAR sc_sigma, alpha;
  static int init_flag=1;
  static SCALAR *sc1, *sc2;

  /******  Initialization   *********/
  if(init_flag)
  {
    SCALAR engy;

    sc1 = (SCALAR *) malloc(1001*sizeof(SCALAR));
    sc2 = (SCALAR *) malloc(NIMAX*sizeof(SCALAR));
    sc1[0] = 9.63e-19; /* sc[en = 1e-4] */
    sc2[0] = 9.63e-19;

    for(i=1; i<1001; i++)
    {
      engy = 0.001*i;
      sc1[i] = 1.0e-20*(6.45 - 0.365*log(engy))*(6.45 - 0.365*log(engy));
    }
    
    for(i=1; i<NIMAX; i++)
    {
      engy = i;
      sc2[i] = 1.0e-20*(6.45 - 0.365*log(engy))*(6.45 - 0.365*log(engy));
    }

    init_flag =0;
  }
  /*--------------------------------------------------------------------*/
   
  if(energy < 1.0) {
    i= 1000*energy;
    alpha= 1000*energy -i;
    sc_sigma = sc1[i] + alpha*(sc1[i+1] - sc1[i]);
  }
  else {
    i= energy;
    if(i < NIMAX-1)
    {
      alpha= energy -i;
      sc_sigma = sc2[i] + alpha*(sc2[i+1] - sc2[i]);
    }
    else
    sc_sigma = sc2[NIMAX-1];
  }

  return(sc_sigma);
}

/*---------------------------------------------------------------------------*/
/*  Ar* + e -> Ar+ + 2e  Ionization of excited states by electron impact
*/
/*---------------------------------------------------------------------------*/

SCALAR asigma6(SCALAR energy)
{
    int i, j, N;
    SCALAR engy, alpha, q;
    static int init_flag=1;
    static SCALAR *ioniz2;
/*
    static SCALAR eng[13] = { 4.16, 5.0, 10., 15., 20., 30., 40., 50., 60.,
                             70., 80., 100, 200};
    static SCALAR sig[13] = { 0.0, 4.6, 8.1, 8.4, 7.8, 7.0, 5.5, 5.1, 4.5,
                             4.2, 3.8, 3.2, 2.0};
*/
    static SCALAR eng[13] = { 0.0, 3.4, 4.21, 4.76, 5.44, 6.8, 10.2, 17.0, 27.2,
                             40.8, 54.4, 81.6, 108.8 };
    static SCALAR sig[13] = { 0.0, 0.0, 7.18, 8.05, 8.37, 8.33, 7.17, 5.07,
                             3.38, 2.40, 1.84, 1.25, 0.951};
    if (init_flag) {
      ioniz2 = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));
      N=12;
      for(i=0; i<NEMAX; i++) {
        engy = i;
        if (engy>eng[N]) {
          ioniz2[i]=sig[N]+(sig[N]-sig[N-1])*(engy-eng[N])/(eng[N]-eng[N-1]);
          if (ioniz2[i]<0) ioniz2[i]=0;
        }
        else if (engy<4.16) ioniz2[i]=0;
        else
          for (j=1;j<=N;j++) if (engy>eng[j-1] && engy<=eng[j])
            ioniz2[i]=sig[j]+(sig[j]-sig[j-1])*(engy-eng[j])/(eng[j]-eng[j-1]);
        ioniz2[i]*=1e-20;
      }
      init_flag=0;
    }

    i=energy;
    if (i<NEMAX-1) {
      alpha=energy-i;
      q=ioniz2[i]+alpha*(ioniz2[i+1] - ioniz2[i]);
    }
    else q = ioniz2[NEMAX-1];

    return(q);
}

/*---------------------------------------------------------------------------*/
/*  Ar + e -> Ar^m + 2e  Excitation to metastable states by electron impact
*/
/*------------------------------------------------------------------LEEHJ----*/

SCALAR asigma7(SCALAR energy)
{
    int i, j, N;
    SCALAR engy, alpha, q;
    static int init_flag=1;
    static SCALAR *meta;
    static SCALAR en[18] = { 11.55, 12.0, 13.0,  15.0, 17.0, 18.0, 20.0, 21.0,
                             22.0, 27.0, 35.0, 40.0, 55.0, 60.0, 82.0, 102.0,
                             122.0, 142.0};
    static SCALAR si[18] = { 0.0, 0.015, 0.07, 0.113, 0.26, 0.3, 0.345, 0.35,
                             0.355, 0.3, 0.188, 0.139, 0.06, 0.05, 0.039,
                             0.035, 0.031, 0.028};

    if (init_flag) {
      meta = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));
      N=17;
      for(i=0; i<NEMAX; i++) {
        engy = i;
        if (engy>en[N]) {
          meta[i]=si[N]+(si[N]-si[N-1])*(engy-en[N])/(en[N]-en[N-1]);
          if (meta[i]<0) meta[i]=0;
        }
        else if (engy<11.55) meta[i]=0;
        else
          for (j=1;j<=N;j++)
            if (engy>en[j-1] && engy<=en[j])
              meta[i]=si[j]+(si[j]-si[j-1])*(engy-en[j])/(en[j]-en[j-1]);
        meta[i]*=1e-20;
      }
      init_flag=0;
    }

    i=energy;
    if (i<NEMAX-1) {
      alpha=energy-i;
      q=meta[i]+alpha*(meta[i+1] - meta[i]);
    }
    else q = meta[NEMAX-1];

    return(q);
}
/*---------------------------------------------------------------------------*/
