/* Monte-Carlo collision routines for particle species with background gas and
** each other.
** LK_asigma1:  Ar + e -> Ar + e      scattering
** LK_asigma2:  Ar + e -> Ar* + e     lumped excitation cross-section (inc 2 radiative and 2 metastable levels)
** LK_asigma3:  Ar + e -> Ar+ + 2e     ionisation from ground state
** LK_asigma4:  Ar+ + Ar -> Ar + Ar+   charge transfer (ions)
*/

#include "pdc1.h"
#include "xgrafix.h"

#define NEMAX    100
#define NIMAX    100

void newvelLK(SCALAR, SCALAR, SCALAR *, SCALAR *, SCALAR *, int);

/* factor for calculating electron mobility */
SCALAR mue_factorLK()
{
  return(0.75); /* isotropic L&K cs set */
}

/*-------------------------------------------------------------*/
/* Monte Carlo scheme for electron and ion-neutrals collisions */

void argonmccLK(int isp)
{
  static int init_flag=1, icorrect=0;
  static SCALAR ecol_extra, icol_extra, ecol_extra2;		
  static SCALAR max_sigmav_e, max_sigmav_i, max_sigmav_e2;	
  static SCALAR col_prob_e, col_prob_i, Cdt, N_ex_max=1;
  
  register int j, i;
  int N, k, nnp, s, index;
  SCALAR etov, dt_correct;
  SCALAR rand_num, v_sqrd, vel, sigma_total, sum_prob;
  SCALAR engy, rengy, frac;
  SCALAR vneutr, vneutth, vneutz;
  SCALAR temp;						

  if (init_flag)
  {
    /*---------------------------------------*/
    /* Calculating the null collision prob.  */
    
    if (ionspecies) 
    {
       ionsp = 1;
       vg_therm = sqrt(T_gas/(Escale[ionsp]));
    }
    if(ecollisional)
    {
      ecolsp = 0;
      max_sigmav_e = 0.0;
      etov = 2.0*ECHARGE/mass[ecolsp];
      for (i=0; i< 10*NEMAX; i++)
      {
	engy = 0.1*i;
	max_sigmav_e= max(max_sigmav_e, sqrt(engy*etov)*(LK_asigma1(engy)+
                      LK_asigma2(engy)+LK_asigma3(engy)));
      }
      col_prob_e = 1.0 -exp(-max_sigmav_e*gas_den*sp_k[ecolsp]*dt);

  #ifdef MPI_1D
      if (my_rank == ROOT){
        printf("Gas density = %lf\n", gas_den);
        printf("Electron coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", 
            col_prob_e, max_sigmav_e, max_sigmav_e*gas_den*dt*sp_k[ecolsp]);
      }
  #else                      
      printf("Gas density = %lf\n", gas_den);
      printf("Electron coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", 
            col_prob_e, max_sigmav_e, max_sigmav_e*gas_den*dt*sp_k[ecolsp]);
  #endif

      max_sigmav_e2 = 0.0;		
/* If radiation transport turned on calculate ionisation of metastables by
electron impact */
      if (RT_flag) 
      {
        for (i=0; i<= 2000; i++) 
        {
          engy = 0.1*i;
          max_sigmav_e2= max(max_sigmav_e2, sqrt(engy*etov)*asigma6(engy));
        }
        Cdt=max_sigmav_e2*dt*sp_k[ecolsp];
      }
    }
    
    if(icollisional)
    {
      icolsp = 1;
      max_sigmav_i = 0.0;
      etov = 2.0*ECHARGE/mass[icolsp];
      for (i=0; i<10*NIMAX; i++)
      {
	engy = 0.1*i;
	max_sigmav_i= max(max_sigmav_i, sqrt(engy*etov)
	                  *(4.0e-19));
      }
      col_prob_i = 1 -exp(-max_sigmav_i*gas_den*sp_k[icolsp]*dt);

  #ifdef MPI_1D
      if (my_rank == ROOT) /* MPI modification; only ROOT prints */
        printf("Ar+ coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", col_prob_i, max_sigmav_i, max_sigmav_i*gas_den*dt*sp_k[icolsp]);
  #else
      printf("Ar+ coll prob = %lf, max(sigma*v) = %lf, nsigmavdt= %lf\n", col_prob_i, max_sigmav_i, max_sigmav_i*gas_den*dt*sp_k[icolsp]);
  #endif

    }
    ecol_extra = icol_extra = ecol_extra2 = 0.5;
    init_flag = 0;
  }
  
  /*------------------------------------------------------------* 
       Electron Collision : Ar* + e  -> Ar+ + 2e  ( by LEEHJ )
   *------------------------------------------------------------*/
  if(RT_flag && ecollisional && isp==ecolsp) 
  {
    dt_correct=gas_den/(N_ex_max*10);
    icorrect++;
    for (N_ex_max=0,j=0;j<nc_RT;j++) N_ex_max=max(N_ex_max, sp_ex[j]+sp_exm[j]);
    if (dt_correct<=icorrect) 
    {
      if(theRunWithXFlag) for (j=0; j<ng; j++) mccrate[5][j] = 0;
      ecol_extra2 += np[ecolsp]* (1-exp(-N_ex_max*icorrect*Cdt));
      N=ecol_extra2;
      ecol_extra2-=N;
  
      nnp = np[ecolsp];
      for(j=0; j< N; j++)
      {
        nnp--;
        index= nnp*frand();
  
        swap(r[ecolsp],index,nnp);
        swap(v_r[ecolsp],index,nnp);
        swap(v_theta[ecolsp],index,nnp);
        swap(v_z[ecolsp],index,nnp);
  
        v_sqrd = v_r[ecolsp][nnp]*v_r[ecolsp][nnp] 
               + v_theta[ecolsp][nnp]*v_theta[ecolsp][nnp]  
               + v_z[ecolsp][nnp]*v_z[ecolsp][nnp];
        engy= Escale[ecolsp]*v_sqrd;
        vel = sqrt(v_sqrd);
        sigma_total = max_sigmav_e2/(vel*vscale);
        rand_num= frand();     

        s=j_pointer(ecolsp,nnp)/nc_ratio; 
        temp=(sp_ex[s]+sp_exm[s])/N_ex_max;
        if (engy>e_miz && rand_num <= (sum_prob=asigma6(engy)/sigma_total)*temp)
        {
        /* first normalize the velocity components by the speed, vel */
          v_r[ecolsp][nnp] /= vel;
          v_theta[ecolsp][nnp] /= vel;
          v_z[ecolsp][nnp] /= vel;

          engy -= e_miz;
          rengy = 2.0*tan(frand()*atan(engy/20.0));
          engy -= rengy;
      
          /*-----------------------------------*/
          /* scatter the created electron	    */

          vel = sqrt(rengy/Escale[ecolsp]);
          k = np[ecolsp];
          v_r[ecolsp][k] = v_r[ecolsp][nnp];
          v_theta[ecolsp][k] = v_theta[ecolsp][nnp];
          v_z[ecolsp][k] = v_z[ecolsp][nnp];
          newvelLK(rengy,vel,&v_r[ecolsp][k],&v_theta[ecolsp][k],&v_z[ecolsp][k],0);
          r[ecolsp][k] = r[ecolsp][nnp];

          /* determine the charge density due to new electron */
        
          frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
          sp_n_mcc[ecolsp][s]  += frac;
          sp_n_mcc[ecolsp][s+1]+= (1.0 - frac); 

          /*-------------------------------------------------------*/
          /* create ion and assign velocities if ion species exist */

          if (ionspecies)
          {
            k = np[ionsp];
            maxwellv(&v_r[ionsp][k],&v_theta[ionsp][k],&v_z[ionsp][k],vg_therm);
            r[ionsp][k] = r[ecolsp][nnp];
  
            /* determine the charge density due to newly created ion */
            sp_n_mcc[ionsp][s]  += frac;
            sp_n_mcc[ionsp][s+1]+= (1.0 -  frac); 
          }

          /*---------------------------------------*/
          /* finally scatter the incident electron */

          vel = sqrt(engy/Escale[ecolsp]);
          newvelLK(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], 
                 &v_z[ecolsp][nnp], 0);

          if(++np[ionsp] >maxnp[ionsp] || ++np[ecolsp] >maxnp[ecolsp])
          {	  
          #ifdef MPI_1D
	          if (my_rank == ROOT)
               printf("mcc(Ionization): too many ions created ");
	       #else
             printf("mcc(Ionization): too many ions created ");
	       #endif
             exit(1);
          }

          if(theRunWithXFlag)
          {
            mccrate[5][s]  += frac/vol[s]/icorrect;
            mccrate[5][s+1]+= (1.0 - frac)/vol[s+1]/icorrect; 
          }

          /*---------------------------------------*/
          /*      Reduce excited state density     */
          k=s/nc_ratio;
          frac=sp_ex[k]/(sp_ex[k]+sp_exm[k]);
          sp_ex[k]-=frac/vol[s];
          sp_exm[k]-=(1-frac)/vol[s];

        }   /* end of test for type of collision */
      }    /* end of for(j=0; j< N; j++)         */
      icorrect=0;
    }     /* end of if (dt_correct<=icorrect)    */
  }  
  /*------------------------------------------------------------* 
             Electron Collision : Ar  + e  
   *------------------------------------------------------------*/
  if(ecollisional && isp==ecolsp)
  {
    if(theRunWithXFlag) {
       for (j=0; j<ng; j++) { 
         mccrate[0][j] = mccrate[1][j] = mccrate[2][j] = mccrate[6][j] =
         Pel[j] = Pie[j] = 0.0;
       }
    
       nu_ex = nu_el = nu_iz = 0; 

    #ifdef POW
       els_loss = 0.0;
    #endif
    }
 #ifdef POW
    else {
      els_loss = 0.0;
      nu_ex = nu_iz = 0;
    }
 #endif

    ecol_extra += np[ecolsp]*col_prob_e;
    N = ecol_extra;
    ecol_extra -= N;
    
    nnp = np[ecolsp];

    for(j=0; j< N; j++)
    {
      nnp--;
      index= nnp*frand();

/* Move colliding particle to end of array, so it isn't chosen twice */
      swap(r[ecolsp],index,nnp);
      swap(v_r[ecolsp],index,nnp);
      swap(v_theta[ecolsp],index,nnp);
      swap(v_z[ecolsp],index,nnp); 

   /*------------------------------------------------------------*/
      /* determine if a collision occurs */
      v_sqrd = v_r[ecolsp][nnp]*v_r[ecolsp][nnp] +v_theta[ecolsp][nnp]*v_theta[ecolsp][nnp]
	     +v_z[ecolsp][nnp]*v_z[ecolsp][nnp];
      engy= Escale[ecolsp]*v_sqrd;
      vel = sqrt(v_sqrd);
      sigma_total = max_sigmav_e/(vel*vscale);
      rand_num= frand();
      
      /*------------------------------------------------------------*/
      /* determine the type of collision and calculate new velocities, etc */
      /*------------------------------------------------------------*/
      /* if the collision is elastic */
      
      if (rand_num <= (sum_prob=LK_asigma1(engy)/sigma_total) )
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
	/* scatter the electron */
	newvelLK(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 1);

        /* collect diagnostics*/
        if(theRunWithXFlag)
        {
          s = j_pointer(ecolsp,nnp);
	  frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
	  temp = frac/vol[s];
	  mccrate[0][s]  += temp;
	  Pel[s] += temp*en_loss;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[0][s+1]+= temp;
	  Pel[s+1] += temp*en_loss;
          nu_el ++;
      #ifdef POW
          els_loss += en_loss;
      #endif
        }
     #ifdef POW
        else els_loss += en_loss;
     #endif     
      }

      /*------------------------------------------------------------*/
      /* Now check for radiative excitation and increase sum_prob by total
         excitation probability */
      
      else if (engy > eexc && rand_num <= (sum_prob += LK_asigma2(engy)/sigma_total))
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
	/* subtract the excitation energy */
	engy -= eexc;
	vel   = sqrt(fabs(engy)/Escale[ecolsp]);
	
	/* scatter the electron */
	newvelLK(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 0);
	
	/* Determine the excitation profile */
	if(theRunWithXFlag)
	{
          s = j_pointer(ecolsp,nnp);
	  frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
	  temp = frac/vol[s];
	  mccrate[1][s]  += temp;
	  Pie[s]  += eexc*temp;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[1][s+1]+= temp; 
	  Pie[s+1]+= eexc*temp; 
          nu_ex ++;
	}
     #ifdef POW
        else nu_ex++;
     #endif     
      }  

      /*------------------------------------------------------------*/
      /* if the collision is ionization, add the released electron and ion first */
      
      else if(engy>eion && rand_num <= (sum_prob += LK_asigma3(engy)/sigma_total))
      {
	/* first normalize the velocity components by the speed, vel */
	v_r[ecolsp][nnp] /= vel;
	v_theta[ecolsp][nnp] /= vel;
	v_z[ecolsp][nnp] /= vel;
	
        /*------------------------------------------------------------*/
	/* subtract the ionisn energy and partition the remaining energy */
	
	engy -= eion;
	rengy = 10.0*tan(frand()*atan(engy/20.0));
	engy -= rengy;
	
        /*------------------------------------------------------------*/
	/* scatter the created electron */
	
	vel = sqrt(rengy/Escale[ecolsp]);
	k = np[ecolsp];
	v_r[ecolsp][k] = v_r[ecolsp][nnp];
	v_theta[ecolsp][k] = v_theta[ecolsp][nnp];
	v_z[ecolsp][k] = v_z[ecolsp][nnp];
	newvelLK(rengy, vel, &v_r[ecolsp][k], &v_theta[ecolsp][k], &v_z[ecolsp][k], 0);
	r[ecolsp][k] = r[ecolsp][nnp];

        /* determine the charge density due to new electron */
        s = j_pointer(ecolsp,nnp);
	frac= (r_grid[s+1] - r[ecolsp][nnp])/dr_n[s];
        sp_n_mcc[ecolsp][s]  += frac;
        sp_n_mcc[ecolsp][s+1]+= (1.0 -  frac); 
	
        /*------------------------------------------------------------*/
	/* create ion and assign velocities if ion species exist */
	
        if (ionspecies)
        {
	  k = np[ionsp];
	  maxwellv(&v_r[ionsp][k], &v_theta[ionsp][k], &v_z[ionsp][k], vg_therm);
	  r[ionsp][k] = r[ecolsp][nnp];

          /* determine the charge density due to newly created ion */
          sp_n_mcc[ionsp][s]  += frac;
          sp_n_mcc[ionsp][s+1]+= (1.0 -  frac); 

        }

        /*------------------------------------------------------------*/
	/* finally scatter the incident electron */
	
	vel = sqrt(engy/Escale[ecolsp]);
	newvelLK(engy, vel, &v_r[ecolsp][nnp], &v_theta[ecolsp][nnp], &v_z[ecolsp][nnp], 0);
	
	if(++np[ionsp] >maxnp[ionsp] || ++np[ecolsp] >maxnp[ecolsp])
	{  
   #ifdef MPI_1D
      if (my_rank == ROOT) printf("mcc(Ionization): too many ions created ");
   #else
      printf("mcc(Ionization): too many ions created ");
   #endif

	 exit(1);
	}
	
        /*------------------------------------------------------------*/
	/* Determine the ionisation profile */
	if(theRunWithXFlag)
	{
	  temp = frac/vol[s];
	  mccrate[2][s]  += temp;
	  Pie[s]  += eion*temp;
	  temp = (1.0 - frac)/vol[s+1]; 
	  mccrate[2][s+1]+= temp; 
	  Pie[s+1]+= eion*temp; 
          nu_iz ++; 
	}
     #ifdef POW
        else nu_iz++;
     #endif

      }   /* end of test for type of collision */
    }     /* end of loop over number colliding electrons*/
  }       /* end of "if (ecollisional)" */

  /*------------------------------------------------------------*/

  if(icollisional && isp==icolsp)
  {
    if(theRunWithXFlag)
      for (j=0; j<ng; j++) mccrate[3][j] = mccrate[4][j] = Pcx[j] = 0.0;

 #ifdef POW
    cx_loss = 0.0;
 #endif

    icol_extra += np[icolsp]*col_prob_i;
    N = icol_extra;
    icol_extra -= N;
    nnp = np[icolsp];

   /*------------------------------------------------------------*/
    for(j=0; j< N; j++)
    {
      nnp--;
      index= nnp*frand();     /* Pick a random ion to collide */
      
/* Move colliding particle to end of array, so it isn't chosen twice */
      swap(r[icolsp],index,nnp);
      swap(v_r[icolsp],index,nnp);
      swap(v_theta[icolsp],index,nnp);
      swap(v_z[icolsp],index,nnp); 

   /*------------------------------------------------------------*/
   /* Check for type of collision */

      maxwellv(&vneutr, &vneutth, &vneutz, vg_therm);
      /* Determine relative velocity btwn neutral and ion */
      v_r[icolsp][nnp] -= vneutr;
      v_theta[icolsp][nnp] -= vneutth;
      v_z[icolsp][nnp] -= vneutz;
      v_sqrd = (v_r[icolsp][nnp]*v_r[icolsp][nnp]
          +v_theta[icolsp][nnp]*v_theta[icolsp][nnp] +v_z[icolsp][nnp]*v_z[icolsp][nnp]);
      engy= Escale[icolsp]*v_sqrd;
      vel = sqrt(v_sqrd);
      sigma_total = max_sigmav_i/(vel*vscale);
      rand_num= frand();
      
       /*------------------------------------------------------------*/
      /* if the collision is charge exchange */
      
      if (rand_num <= 4.0e-19/sigma_total)
      {
	v_r[icolsp][nnp] = v_theta[icolsp][nnp] = v_z[icolsp][nnp] = 0.0;
	
     /* Count number of charge exchange collisions */ 
        if(theRunWithXFlag)
	{
          s = j_pointer(icolsp,nnp);
	  frac= (r_grid[s+1] - r[icolsp][nnp])/dr_n[s];
	  temp = frac/vol[s]/sp_k[icolsp];
	  mccrate[3][s]  += temp;
	  Pcx[s] += engy*temp;
	  temp = (1.0 - frac)/vol[s+1]/sp_k[icolsp];
	  mccrate[3][s+1]+= temp; 
	  Pcx[s+1] += engy*temp;
       #ifdef POW
	  cx_loss += engy;
       #endif
	}
    #ifdef POW
        else cx_loss += engy;
    #endif
      }  /* end of "else rand_num < sigma + dsig" loop */

    /* Add neutral velocity back to ion velocity */

      v_r[icolsp][nnp] += vneutr;
      v_theta[icolsp][nnp] += vneutth;
      v_z[icolsp][nnp] += vneutz;
    }    /* end of loop of colliding particles */
  }      /* end of if icollisional loop */

}       /* end of mcc subroutine */

/*************************************************************
**  Calculates new velocities for particles which have 
**  just made a collision
**************************************************************/
void newvelLK(SCALAR energy, SCALAR vel, SCALAR *vr, SCALAR *vth, SCALAR *vz, int e_flag)
{
  SCALAR phi1, cosphi, sinphi, coschi, sinchi, up1, up2, up3;
  SCALAR mag, r11, r12, r13, r21, r22, r23, r31, r32, r33;
  SCALAR tmp;

  coschi = 1.0 - 2.0*frand();  /* isotropic scattering angle */
  sinchi= sqrt(1. - coschi*coschi);
  
  phi1  = TWOPI*frand();
  cosphi= cos(phi1);
  sinphi= sin(phi1);
  
  if (e_flag) {
     tmp = 2.0*mass[ecolsp]*(1-coschi)/mass[ionsp];
     vel *= sqrt(1.0 - tmp);
     en_loss = energy*tmp;
  }
  
  r13 = *vr;
  r23 = *vth;
  r33 = *vz;
  
  if(r33 == 1.0) { up1= 0;  up2= 1;  up3= 0; }
  else           { up1= 0;  up2= 0;  up3= 1; }
  
  r12 = r23*up3 -r33*up2;
  r22 = r33*up1 -r13*up3;
  r32 = r13*up2 -r23*up1;
  mag = sqrt(r12*r12 + r22*r22 + r32*r32);
  
  r12/= mag;
  r22/= mag;
  r32/= mag;
  
  r11 = r22*r33 -r32*r23;
  r21 = r32*r13 -r12*r33;
  r31 = r12*r23 -r22*r13;
  
  *vr= vel*(r11*sinchi*cosphi +r12*sinchi*sinphi +r13*coschi);
  *vth= vel*(r21*sinchi*cosphi +r22*sinchi*sinphi +r23*coschi); 
  *vz= vel*(r31*sinchi*cosphi +r32*sinchi*sinphi +r33*coschi);
}

/*------------------------------------------------------------*/
/*  Total e- Scattering Cross-section                            */
/*-------------------------------------------------------------*/
SCALAR LK_asigmaT(SCALAR energy)
{
  return(LK_asigma1(energy) + LK_asigma2(energy) + LK_asigma3(energy));
 }
  

/*---------------------------------------------------------------------------*/
/* XSections for Argon-Electron Collisions                      */
/*------------------------------------------------------------*/
/*  e + Ar -> e + Ar  Elastic                                  */
/*-------------------------------------------------------------*/
SCALAR LK_asigma1(SCALAR energy)
{
  int i;
  SCALAR els_sigma, alpha;
  static int init_flag=1;
  static SCALAR *elastic;

  /*-----  Initialization   --------*/
  if(init_flag) {
    SCALAR engy;
    SCALAR amp_1 = 1.59e-19;

    elastic = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));
  
    for(i=0; i<NEMAX; i++)
    {
      engy = i;
      if (engy <= eexc) elastic[i]= amp_1*engy/eexc;
      else elastic[i] = amp_1*sqrt(eexc/engy);
    }

    init_flag =0;
  }
/*------------------------------------------------------------------------*/
  i= energy;
  if(i < NEMAX-1)
  {
    alpha= energy -i;
      els_sigma = elastic[i] + alpha*(elastic[i+1] - elastic[i]);
    }
    else
      els_sigma = elastic[NEMAX-1];

  return(els_sigma);
}
/*------------------------------------------------------------------------*/
/* Momentum transfer cross-section - calculate from the elastic x-s       */
/*------------------------------------------------------------------------*/
SCALAR LK_asigma1a(SCALAR energy)
{
  SCALAR q_mtm;  
  q_mtm = LK_asigma1(energy);
  return(q_mtm);
}
/*---------------------------------------------------------------------------*/
/*  e + Ar -> e + Ar  Excitation    */
/*---------------------------------------------------------------------------*/
SCALAR LK_asigma2(SCALAR energy)
{
  int i;
  SCALAR exc_sigma, alpha;
  static int init_flag=1;
  static SCALAR *excit;

  /*-----  Initialization   --------*/
  if(init_flag)
  {
       SCALAR x;
       SCALAR amp = 1.56e-20;

       excit = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

       for(i=0; i<NEMAX; i++)
       {
            x = (SCALAR)i/eexc;     /* x = energy/eexc */
            if(x < 1.0)
                excit[i] = 0;
            else
                excit[i] = amp*log(x)/x;
       }
       init_flag =0;
  }
  /*--------------------------------------------------------------------*/

  i= energy;
  if(i < NEMAX-1)
  {
    alpha= energy -i;
    exc_sigma = excit[i] + alpha*(excit[i+1] - excit[i]);
  }
  else
    exc_sigma = excit[NEMAX-1];

  return(exc_sigma);

}

/*---------------------------------------------------------------------------*/
/*  e + Ar -> e + e + Ar+  Ion.     */
/*---------------------------------------------------------------------------*/

SCALAR LK_asigma3(SCALAR energy)
{
  int i;
  SCALAR ion_sigma, alpha;
  static int init_flag=1;
  static SCALAR *ioniz;

  /*-----  Initialization   --------*/
  if(init_flag)
  {
    SCALAR amp=3.18e-20, x;

    ioniz = (SCALAR *) malloc(NEMAX*sizeof(SCALAR));

    for(i=0; i<NEMAX; i++)
    {
      x = (SCALAR)i/eion;    /* x = energy/eion */
      if (x < 1.0)
          ioniz[i] = 0;
      else
          ioniz[i] = amp*log(x)/x;
    }
    init_flag =0;
  }
  /*--------------------------------------------------------------------*/
  i= energy;
  if(i < NEMAX-1)
  {
    alpha= energy -i;
    ion_sigma = ioniz[i] + alpha*(ioniz[i+1] - ioniz[i]);
  }
  else
    ion_sigma = ioniz[NEMAX-1];

  return(ion_sigma);
}
/*---------------------------------------------------------------------------*/
/* Argon Ion-neutral Collision Cross-sections                                */
/*---------------------------------------------------------------------------*/
/*  Ar + Ar+ -> Ar+ + Ar  Charge Exchange       LK_asigma4 = 4.0e-19 */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/*  Ar + Ar+ -> Ar + Ar+   No Elastic Scattering in K&L Model               */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/*  Ar* + e -> Ar+ + 2e  Ionization of excited states by electron impact     */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/*  Ar + e -> Ar^m + 2e  Excitation to metastable states by electron impact */
/* Not included in K&L cross-sections                                       */
/*------------------------------------------------------------------LEEHJ----*/

