
/*******************************************************************/
/*	This set of routines does the charge weighting to the      */
/*	grid, calculates the potentials and fields at the grid,    */
/*	interpolates the fields back to the charged species and    */
/*	then moves them. 					   */
/*                                                                 */
/*	* the weighting can be linear or volume                    */
/*      * potentials are solved using a backwards tridiagonal      */
/*	matrix method.    					   */
/*                                                                 */
/*      It is based on the routine pflmv.c, but has been           */
/*      re-written                                                 */
/*      1) calculations all use normalised parameters and only     */
/*         diagnostics are re-normalised  (to reduce calc time)    */
/*      2) A ramp is now available for the ac voltage              */
/*      3) Simulation particle numbers can be re-scaled to keep    */
/*         them within specified parameters                        */
/*      4) The charge density is smoothed using a one-to-one       */
/*         filter to noise in the fields                           */
/******************************************************************/
#include "pdc1.h"
#include <xgrafix.h>

SCALAR source();
void circuit1(), circuit2(), circuit3(), circuit4(),
circuit5(), circuit6(), circuit7(); 
void  (*circuitptr)();
void DigitalFilter(SCALAR *, SCALAR *, int, int);

SCALAR *xp, *a_phi, *b_phi, *c_phi;
SCALAR ke_sum1, ke_old; 
SCALAR Iz_norm, mu_norm;
SCALAR vz_sum, sv_sum;


#ifdef MPI_1D
SCALAR sum[2], sum_tot[2];
#endif


/***************************************************************/
void gather(int isp)
/* Accumulate charge density on grid */
{
  int ii, j, s;
  SCALAR frac;
  SCALAR vr_sq, vt_sq, vz_sq, energy, dr_en, vel, sv_avg, ur, ut, uz, ur_sq, ut_sq, uz_sq, current; //u_sq
  
/* Set current, ke profile and fe diagnostics to 0 */
    if (theRunWithXFlag)
    {
      for (ii=0; ii<ng; ii++) {
	j_r[isp][ii]=j_z[isp][ii]=j_t[isp][ii]=0;
	ke_r[isp][ii]=ker_r[isp][ii]=ket_r[isp][ii]=kez_r[isp][ii]=0; 
      }
      ke_sp[isp] = T_sp[isp] = u_s[isp] = ur_s[isp] = ut_s[isp] = uz_s[isp] = 0.0;
      for (ii=0; ii<=nbin_mid[isp]; ii++) fe_mid[isp][ii]=0.0;
      sv_avg = 0;
      ur = ut = uz = 0; /* set drift components to zero */
    }

/* Calculate total density at t-1 due to position + ioniz colls and 0
sp_n_mcc and sp_n_k */
  for (ii=0; ii<ng; ii++) 
  {
     sp_n_0[isp][ii] = sp_n_k[isp][ii] + sp_n_mcc[isp][ii];
     sp_n_mcc[isp][ii]= sp_n_k[isp][ii]= 0.0;
  }

/* Loop over number of particles */
    for (ii=0; ii<np[isp]; ii++) 
    {
/* check that particle is between electrodes */
      if (r[isp][ii]>=r0 && r[isp][ii]<r1) 
      {
        j = j_pointer(isp,ii);

        /* linear weighting */
	frac = (r_grid[j+1] - r[isp][ii])/dr_n[j];
	sp_n_k[isp][j] += frac;
	sp_n_k[isp][j+1] += (1.0 - frac);

/*   Calculate total ke of species, store kinetic energy and current at grid
cells for profile diagnostics and determine electron energy distribution in
plasma  */
        if (theRunWithXFlag)
        {
           j_r[isp][j] += frac*v_r[isp][ii];
           j_r[isp][j+1] += (1.0 - frac)*v_r[isp][ii];
           j_t[isp][j] += frac*v_theta[isp][ii];
           j_t[isp][j+1] += (1.0 - frac)*v_theta[isp][ii];
           j_z[isp][j] += frac*v_z[isp][ii];
           j_z[isp][j+1] += (1.0 - frac)*v_z[isp][ii];
	   current = j_r[isp][ii]+j_t[isp][ii]+j_z[isp][ii];
	   current_sp[isp] += current;
           vr_sq = SQUARE(v_r[isp][ii]);
           vt_sq = SQUARE(v_theta[isp][ii]);
           vz_sq = SQUARE(v_z[isp][ii]);
           energy = vr_sq + vt_sq + vz_sq;
           ke_sp[isp] += energy;
	   ur += v_r[isp][ii]/np[isp];
	   ut += v_theta[isp][ii]/np[isp];
	   uz += v_z[isp][ii]/np[isp];
   	   dr_en = (ur*ur + ut* ut +uz*uz);
	   u_s[isp] += dr_en;
	   T_sp[isp] += (ke_sp[isp]/np[isp] - u_s[isp]/np[isp])*2/3;
           ke_r[isp][j] += frac*energy;
           ke_r[isp][j+1] += (1.0 - frac)*energy;
           ker_r[isp][j] += frac*vr_sq;
           ker_r[isp][j+1] += (1.0 - frac)*vr_sq;
           ket_r[isp][j] += frac*vt_sq;
           ket_r[isp][j+1] += (1.0 - frac)*vt_sq;
           kez_r[isp][j] += frac*vz_sq;
           kez_r[isp][j+1] += (1.0 - frac)*vz_sq;
	   
           if ((r[isp][ii] - rs_mid[isp])*(rf_mid[isp] - r[isp][ii]) >= 0)
           {
              s = (energy - emin_mid[isp])/de_mid[isp];
              if (0 <= s && s < nbin_mid[isp]) fe_mid[isp][s] += nc2p;
           }

/* Calculate average coll freq if NOT using axial current as source */
           if (isp==0 && src != 'J' && src != 'j')
           {
             vel = sqrt(energy);
             energy *= Escale[0];
             sv_avg += (*sigmaTptr)(energy)*vel;
           }
        } /* end of diagnostic loop */
      } /* end of "if (r[isp][i] > r0" loop */
    }   /* end of loop over all particles */

    for (ii=0; ii<ng; ii++) {
      //   T_sp[isp][ii] = (ke_sp[isp]/np[isp] - dr_en/np[isp])*2/3;
    }

    
/* Divide ke_r by particle number to get average ke per particle */
    if(theRunWithXFlag) {

      /*      printf("isp=%d ur=%G ut=%G uz=%G\n",isp, ur*vscale, ut*vscale, uz*vscale);*/

      for (ii=0; ii<ng; ii++) {
        if (sp_n_k[isp][ii])
        {
 	  u_r[isp][ii] = j_r[isp][ii]/sp_n_k[isp][ii];
	  u_t[isp][ii] = j_t[isp][ii]/sp_n_k[isp][ii];
	  u_z[isp][ii] = j_z[isp][ii]/sp_n_k[isp][ii];
	  ur_sq = u_r[isp][ii]*u_r[isp][ii];
	  ut_sq = u_t[isp][ii]*u_t[isp][ii];
	  uz_sq = u_z[isp][ii]*u_z[isp][ii];

	  //Ttotal2[isp][ii] = ke_r[isp][ii] - SQUARE(j_r[isp][ii]) - SQUARE(j_t[isp][ii]) - SQUARE(j_r[isp][ii]);
	  //Ttotal2[isp][ii] *= Escale[isp]*2/3;

	  // 	  ur_sq *= sp_n_k[isp][ii];
	  //	  ut_sq *= sp_n_k[isp][ii];
	  //	  uz_sq *= sp_n_k[isp][ii];

	  //           ke_r[isp][ii] /= sp_n_k[isp][ii];
       	  //           ker_r[isp][ii] /= sp_n_k[isp][ii];
	  //           ket_r[isp][ii] /= sp_n_k[isp][ii];
	  //           kez_r[isp][ii] /= sp_n_k[isp][ii];

	   Ttotal[isp][ii] = ke_r[isp][ii]/sp_n_k[isp][ii] - ur_sq-ut_sq-uz_sq;
	   Ttotal[isp][ii] *= Escale[isp]*2/3;
	   //Ttotal2[isp][ii] = Ttotal[isp][ii] * sp_n_k[isp][ii];//*nc2p*nc2p * sp_n_k[isp][ii];
	   
	   //	   Tr[isp][ii] = (ker_r[isp][ii] - ur_sq)*Escale[isp]*2/3;
	   //	   Tt[isp][ii] = (ket_r[isp][ii] - ut_sq)*Escale[isp]*2/3;
	   //	   Tz[isp][ii] = (kez_r[isp][ii] - uz_sq)*Escale[isp]*2/3;
	   Dr_r[isp][ii] = ur_sq+ut_sq+uz_sq;
	   //	   Drr_r[isp][ii] = ur_sq;
	   //	   Drt_r[isp][ii] = ut_sq;
	   //	   Drz_r[isp][ii] = uz_sq;
	   Dr_r[isp][ii] *= Escale[isp];
	   //	   Drr_r[isp][ii] *= Escale[isp];
	   //	   Drt_r[isp][ii] *= Escale[isp];
	   //	   Drz_r[isp][ii] *= Escale[isp];
	   ur_sq *= Escale[isp];
	   ut_sq *= Escale[isp];
	   uz_sq *= Escale[isp];

        }
	else ke_r[isp][ii] = ker_r[isp][ii] = ket_r[isp][ii] = kez_r[isp][ii]
	       = Ttotal[isp][ii] /*= Ttotal2[isp][ii] */=Tr[isp][ii] = Tt[isp][ii] = Tz[isp][ii] 
	       = Dr_r[isp][ii] =Drr_r[isp][ii] =Drt_r[isp][ii] =Drz_r[isp][ii] 
	       = 0;

		j_r[isp][ii] /= vol[ii];
		j_t[isp][ii] /= vol[ii];
		j_z[isp][ii] /= vol[ii];
      }
      
     /* Calculate theoretical electon mobility */
     if (isp==0 && src !='J' && src !='j') mu_e = mu_norm*np[0]/sv_avg;

     } 
}





/******************************************************************/
/*  The init_rho rouutine is used to determine the charge density */
/*  at the initial (t=0) step                                     */
/******************************************************************/
void init_rho() {
  int j, isp;

  for(isp=0; isp<nsp; isp++) {
    it[isp] = 0;
    k_count[isp] = 0;
    gather(isp);
    for (j=0; j<ng; j++) {
      sp_n[isp][j]= sp_n_k[isp][j];
      sp_n_mcc[isp][j]= 0.0;
    }
  }
}




/***************************************************************/
void fields() {
  int isp, ii;

/* Find rho(x) = total charge density...
     ..first put in background (rigid) charge density */
  for (ii=0; ii<ng; ii++) rho[ii]= rhoback;

/* ... now add in species charge densities */
  for (isp=0; isp<nsp; isp++)
    for (ii=0; ii<ng; ii++) 
      #ifdef MPI_1D	
        rho[ii] += q_ratio[isp]*sp_n_tot[isp][ii];
      #else
        rho[ii] += q_ratio[isp]*sp_n[isp][ii];
      #endif

/* Digitally smooth the net charge density */
  if (nsmoothing > 0) 
  {
	 for (ii = 0; ii<ng; ii++) rho_unfilt[ii] = rho[ii];
	 DigitalFilter(rho_unfilt, rho, ng, nsmoothing);
  }

  /* determine convective (particle) current to powered electrode */
  i_conv = backgrnd_i;                                /* first background current */
  #ifdef MPI_1D
    for (isp=0; isp< nsp; isp++) i_conv += iwall_tot[isp];  /* now species current */
   #else
    for (isp=0; isp< nsp; isp++) i_conv += iwall[isp];  /* now species current */
   #endif 

  /* Solve a tridiagnal matrix to get potential at time t*/
  (*circuitptr)();  /* pointer to appropriate circuit calc */ 
  
  /* Now calculate electric field at grid from potentials */
  for (ii=1; ii<nc; ii++) efield[ii] = (phi[ii-1] - phi[ii+1])/(dr_n[ii-1] + dr_n[ii]); 
  if (r0 > 0) efield[0] = ((r0+0.5*dr_n[0])*(phi[0]-phi[1]) - 0.5*rhon*rho[0]*vol[0])/r0;
  else efield[0] = 0.0;
  efield[nc]=((r1-0.5*dr_n[nc-1])*(phi[nc-1]-phi[nc])/dr_n[nc-1] + 0.5*rhon*rho[nc]*vol[nc])/r1;

}	/* End of fields() */









/**************************************************************/
/* Determines time-dependent source voltage. Can have         */
/*    1) dc voltage with a linear ramp                        */
/*    2) ac voltage with an exponential ramp                  */
/*    3) a combination of both                                */
/* It doesn't do a cosine ramp for the dc voltage anymore     */ 
/**************************************************************/
SCALAR source()
{
  SCALAR Sdc, Sac;
  SCALAR Sdc2, Sac2;

  Sdc = dcbias;
  Sac = acbias*sin(atime*w0 + theta0);

  if (ramped)
  {
    if(dcbias && atime < risetime) Sdc = (dc_ramp*atime);
    if(ac_ramp > 0.0) Sac *= exp(-ac_ramp/atime);
    if(ac_ramp < 0.0) Sac *= exp(atime/ac_ramp);
  }
  if((src == 'W') ||(src == 'K'))
    {
      Sdc2 = dcbias2;
      Sac2 = acbias2*sin(atime*w02 + theta02);
      
      if (ramped2)
        {
          if(dcbias2 && atime < risetime2) Sdc = (dc_ramp2*atime);
          if(ac_ramp2 > 0.0) Sac *= exp(-ac_ramp2/atime);
          if(ac_ramp2 < 0.0) Sac *= exp(atime/ac_ramp2);
        }
      
      return (Sac2 + Sdc2 + Sac + Sdc);
      
    }

  return (Sac + Sdc);
} /*  End of source()*/








/********************************************************************/
/* Smoothing the array using a digital filter analogous to 1-2-1    */
/* method used in planar system with the proper boundary conditions */ 
/* to conserve charge. Note that nSmoothing is only used on first   */
/* pass thru to set up coefficients for multipass smoothing         */
/********************************************************************/
void DigitalFilter(SCALAR *source, SCALAR *dest, int N, int nSmoothing)
{
  static int init = 0;
  static SCALAR *alpha, **A;
  int j, c, d;
  SCALAR Ajd, Ajdm1;
  if (nSmoothing < 1) return;
  N--; /* last element index: 0...N */

  /** Initialize arrays on first use of filter **/
  if (init==0)  {
    int i;
    init = 1;
    alpha = (SCALAR *) malloc((N+1)*sizeof(SCALAR));
    alpha[0] = 0.5;
    alpha[1] = alpha[0]*vol[0]/vol[1];
    for (j=1; j<N; j++)
      alpha[j+1] = (2*alpha[j]*vol[j] - alpha[j-1]*vol[j-1])/vol[j+1];
    A = (SCALAR **) malloc((N+1)*sizeof (SCALAR *));
    for (j=0; j<=N; j++) {
      A[j] = (SCALAR *) calloc((2*nSmoothing + 1),sizeof(SCALAR));
      /* intitial values for one pass */
      for (c=max(0,j-1); c<=min(N,j+1); c++){
	d = c - j + nSmoothing;
	if (j==c) 
	  if (j==0 || j==N) A[j][d] = 1 - alpha[j];
	  else A[j][d] = 1 - 2*alpha[j];
	else A[j][d] = alpha[j];
      }
    }
    for (i=1; i<nSmoothing; i++) {
      for (j=0; j<=N; j++) {
	Ajd = A[j][0];
	for (c=max(0,j-nSmoothing); c<=min(N,j+nSmoothing); c++){
	  d = c - j + nSmoothing;
	  Ajdm1 = Ajd; /* shift */
	  Ajd = A[j][d]; /* save new element */
	  if (c == 0) A[j][d] = Ajd*(1-alpha[c]) + A[j][d+1]*alpha[c+1];
	  else if (c==N) A[j][d] = Ajdm1*alpha[c-1] + Ajd*(1-alpha[c]);
	  else A[j][d] = Ajdm1*alpha[c-1] + Ajd*(1-2*alpha[c]) + A[j][d+1]*alpha[c+1];
	}
      }
    }
  }
  for (j=0; j<=N; j++) {
    dest[j] = 0;
    for (c=max(0,j-nSmoothing); c<=min(N,j+nSmoothing); c++){
      d = c - j + nSmoothing;
      dest[j] += source[c]*A[j][d];
    }
  }
}  /*  End of DigitalFilter  */







/***************************************************************/
/*    Initialise parameters for calculating the field solve    */
/*    using tridiagonal matrix method                          */
/***************************************************************/
void field_init()
{
  int i, k;
  SCALAR indn, capn, resn;
  SCALAR jz_norm, pz_norm, r_avg;

/****** Create space for alphas, as, cs, etc */
  a_phi = (SCALAR  *)malloc(nc*sizeof(SCALAR));
  b_phi = (SCALAR  *)malloc(nc*sizeof(SCALAR));
  c_phi = (SCALAR  *)malloc(nc*sizeof(SCALAR));
  xp = (SCALAR *)malloc(nc*sizeof(SCALAR));

  if(!a_phi || !b_phi || !c_phi || !xp )
  {
  #ifdef MPI_1D
    if(my_rank == ROOT)
      printf("start() : malloc() a, c, xp returned NULL");
  #else
    printf("start() : malloc() a, c, xp returned NULL");
  #endif
    exit(-1);
  }

/* normalise circuit parameters */
     capn = extc/(ptopn*ECHARGE*nc2p);
     indn = extl*ptopn*ECHARGE*nc2p/dt/dt;
     resn = extr*ptopn*ECHARGE*nc2p/dt;

/**Calc constants for pot calc */
  for (i=1; i<nc; i++)
  {
     r_avg = (dr_n[i] + dr_n[i-1])*0.5;
     a_phi[i]=1.0/dr_n[i-1]/r_avg - dr_n[i]/(r_grid[i]*2.0*r_avg*dr_n[i-1]);
     b_phi[i]=2.0/dr_n[i]/dr_n[i-1] - (dr_n[i] - dr_n[i-1])/r_grid[i]/dr_n[i]/dr_n[i-1];
     c_phi[i]=1.0/r_avg/dr_n[i] + dr_n[i-1]/(r_grid[i]*2.0*r_avg*dr_n[i]);
  }
  c_phi[0] = 1.0;

/* Calc mu_norm if diagnostics turned on */
  if(theRunWithXFlag)
    mu_norm =  (*mueFacPtr)()*ECHARGE/(mass[0]*gas_den*vscale);

 /*---------------------------------------------*/
 /* Normalise external circuit parameters       */
  if(src== 'I' || src== 'i' || src=='K')
  {
      acbias *=dt/ECHARGE/nc2p;
      dcbias *=dt/ECHARGE/nc2p;
      acbias2 *=dt/ECHARGE/nc2p;
      dcbias2 *=dt/ECHARGE/nc2p;
  } else if (src=='V' || src=='v' || src=='W')
  {
      acbias *=ptopn;
      dcbias *=ptopn;
      acbias2 *=ptopn;
      dcbias2 *=ptopn;
  }
  else if (src=='J' || src=='j')
  {
    jz_norm = height*dt/(nc2p*mass[0]*vscale);
    mu_norm = (*mueFacPtr)()*ECHARGE/(mass[0]*gas_den*vscale);
    dcbias *= jz_norm;
    acbias *= jz_norm;
    circuitptr= circuit5;
    Iz_norm = charge[0]*nc2p*vscale/height;
    
    ke_sum1 = 0;
    for (i=0; i<np[0]; i++)
        ke_sum1 += v_r[0][i]*v_r[0][i] + v_theta[0][i]*v_theta[0][i] + v_z[0][i]*v_z[0][i];
     ke_old = ke_sum1;
  }
  else if (src=='P' || src=='p')  {
/* Convert to normalised power */
     pz_norm = ptopn*dr/height;
     acbias *= pz_norm;
     dcbias *= pz_norm;
     circuitptr= circuit7;
     Iz_norm = charge[0]*nc2p*vscale/height;

     ke_sum1 = 0;
     for (i=0; i<np[0]; i++)
        ke_sum1 += v_r[0][i]*v_r[0][i] + v_theta[0][i]*v_theta[0][i] + v_z[0][i]*v_z[0][i];
     ke_old = ke_sum1;
  }
  else if (src=='E' || src=='e')  { 
    dcbias *= ptopn*dr;
    acbias *= ptopn*dr;
    circuitptr= circuit6;
  }

  if (r0 == 0.0) { /* no inner conductor */ 
    xp[0] = 1.0;
    k = 1;
  }
  else if(extc < 1e-30)  /* Open Circuit, when C tends to 0.0 */ 
  {
    if(fabs(dcbias)+fabs(acbias) >0.0)
    {
      #ifdef MPI_1D
	if (my_rank == ROOT)
	  printf("START:The active external source is ignored since C<1e-30\n");
      #else
	printf("START:The active external source is ignored since C<1e-30\n");
      #endif
       
    }
	circuitptr = circuit1;
	k=1;
	xp[0]=1.0;
  } 
  else if(src== 'I' || src== 'i' || src =='K')       
/* When current source is applied */
  {
    circuitptr= circuit2;
    k=1;
    xp[0]= 1.;
  }
  else if(indn <1e-30 && resn <1e-30 &&        /* Short Circuit */
         (extc*log(r1/r0)/(TWOPI*height*epsilon)) >= 1e5) 
  {
    circuitptr = circuit3;
    k=2;
    xp[1]=0.5;
  }
  else  /* The general case with external voltage source */
  { 
     circuitptr = circuit4;

     alpha0 = 1.5*resn + 2.25*indn + 1.0/capn;
     alpha1 = 2.0*resn + 6.0*indn;
     alpha2 = -0.5*resn - 5.5*indn;
     alpha3 = 2.0*indn;
     alpha4 = -0.25*indn;

     k=1; 
     xp[0] = 1.0/(1.0 + 0.5*rhon/(r0 + 0.5)/alpha0);
  }
  for (i=k;i<nc;i++) xp[i] = 1.0/(b_phi[i] - a_phi[i]*c_phi[i-1]*xp[i-1]);

}       /* end field_init */





/*************************/
/*  1: Open circuit case */
/*************************/

void circuit1()

{
  int ii;
  SCALAR yp[nc];

  oldsigma = sigma;
  sigma += i_conv/r0;

  yp[0] = xp[0]*0.5*rhon*(sigma*r0 + rho[0]*(r0 + 0.25))/(r0+0.5);
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];
}





/*********************/
/* 2: Current source */
/*********************/

void circuit2()
{
  int ii;
  SCALAR temp;
  SCALAR yp[nc];
  
  /* Calculate charge density on electrode - second order method  */
  I_rf = source();
  temp= sigma;
  sigma = (4.*sigma -oldsigma +2.0*(i_conv+I_rf)/r0)/3.0;
  oldsigma= temp;

  /* Calculate charge density on electrode */
  yp[0] = 0.5*rhon*xp[0]*(sigma*r0 + rho[0]*(r0 + 0.25))/(r0 + 0.5);
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];

}




/*************************/
/* 3: Short circuit case */
/*************************/

void circuit3()
{
  int ii;
  SCALAR yp[nc];

  phi[0] = source();
  yp[1] = xp[1]*(a_phi[1]*phi[0] + rhon*rho[1]);
  for (ii=2; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];

  oldsigma = sigma;
  sigma = ((r0+0.5)*(phi[0]-phi[1])*2.0/rhon - (r0+0.25)*rho[0])/r0;
  I_rf = (sigma-oldsigma)*r0 - i_conv;
}




/*******************************************************/
/* 4: General voltage source case w RLC series circuit */
/*******************************************************/

void circuit4()
{
  int ii;
  SCALAR k_dash;
  SCALAR yp[nc];
  
  k_dash = alpha1*q_0 + alpha2*q_1 +alpha3*q_2 +alpha4*q_3;

  yp[0] = xp[0]*0.5*rhon*(r0*sigma + i_conv + (k_dash + source())/alpha0 - q_0
           + rho[0]*(r0 + 0.25))/(r0 + 0.5);
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];

  q_3= q_2;
  q_2= q_1;
  q_1= q_0;
  q_0 = (k_dash + source() -phi[0])/alpha0;
  
  I_rf = q_0 - q_1;
  oldsigma = sigma;
  sigma += (i_conv + I_rf)/r0;
}



 
/****************************************************************************/
/* 5: Case with no inner conductor and axial current supplied               */
/* This version calculates the electron mobility from the electron momentum */
/* transfer collision frequency.                                            */
/****************************************************************************/

void circuit5()
{
  int ii;
  static int skip = 1;
  SCALAR yp[nc];
  SCALAR energy, vel, Ne;

  /* calculate average collision frequency every skip timesteps */
  if (it[0]%skip == 0)
  {  
  #ifdef MPI_1D
     if (my_rank == ROOT) {
       if ( fabs(ke_old - ke_sum1)/ke_old < 0.0001) skip *= 2;
       else if ( fabs(ke_old - ke_sum1)/ke_old  > 0.1 && skip > 1) skip /= 2;
     }
     MPI_Bcast(&skip, 1, MPI_INT, ROOT, MPI_COMM_WORLD);
  #else
     if ( fabs(ke_old - ke_sum1)/ke_old < 0.0001) skip *= 2;
     else if ( fabs(ke_old - ke_sum1)/ke_old  > 0.1 && skip > 1) skip /= 2;
  #endif
     ke_old = ke_sum1;
     sv_sum = 0.0;
     ke_sum1 = 0.0;
     vz_sum = 0.0;
     for (ii=0; ii<np[0]; ii++)
     {
        energy = v_r[0][ii]*v_r[0][ii] + v_theta[0][ii]*v_theta[0][ii] + v_z[0][ii]*v_z[0][ii];
        ke_sum1 += energy;
        vz_sum += v_z[0][ii];
        vel = sqrt(energy);
        energy *= Escale[0];
	sv_sum += (*sigmaTptr)(energy)*vel;
     }
#ifdef MPI_1D
     sum[0] = sv_sum;
     sum[1] = vz_sum;
     MPI_Allreduce(sum, sum_tot, 2, MPI_FLOAT, MPI_SUM, MPI_COMM_WORLD);
     I_z = Iz_norm*sum_tot[1];
#else
     I_z = Iz_norm*vz_sum;
#endif
  }
	    
#ifdef MPI_1D
  MPI_Allreduce(np, np_tot, NSMAX, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
  Ne = (SCALAR) np_tot[0];
  mu_e = mu_norm*Ne/sum_tot[0];
  E_z = source()/(mu_e*Ne);
  /*if (my_rank == 0) printf("time=%e, E_z=%lf, Iz=%lf, Ne=%lf, mu_e=%lf\n", atime, E_z/(ptopn*dr), I_z, Ne, mu_e);*/
#else
  Ne = (SCALAR) np[0];
  mu_e = mu_norm*Ne/sv_sum;
  E_z = source()/(mu_e*Ne);
 /* printf("time=%e, E_z=%lf, Iz=%lf, Ne=%lf, mu_e=%lf\n", atime, E_z/(ptopn*dr), I_z, Ne, mu_e);*/
#endif
  yp[0] = rhon*rho[0]*vol[0];
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];

} /* end of circuit5 */






/***************************************************************/
/* 6: Case with no inner conductor and axial field supplied    */
/***************************************************************/
void circuit6()
{
  int ii;
  SCALAR yp[nc];
  
  E_z = source();
  yp[0] = vol[0]*rhon*rho[0];
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];
}





/***************************************************************/
/* 7: Case with no inner conductor and axial power supplied    */
/***************************************************************/
void circuit7()
{
  int ii;
  static int skip=1;
  SCALAR yp[nc];
  SCALAR energy, vz_sum;
#ifdef MPI_1D
  SCALAR vz_sum_tot;
#endif

  /* calculate I_z every skip timesteps */
  if (it[0]%skip == 0)
  {  
  #ifdef MPI_1D
     if (my_rank == ROOT) {
       if ( fabs(ke_old - ke_sum1)/ke_old < 0.0001) skip *= 2;
       else if ( fabs(ke_old - ke_sum1)/ke_old  > 0.1 && skip > 1) skip /= 2;
     }
     MPI_Bcast(&skip, 1, MPI_INT, ROOT, MPI_COMM_WORLD);
  #else
     if ( fabs(ke_old - ke_sum1)/ke_old < 0.0001) skip *= 2;
     else if ( fabs(ke_old - ke_sum1)/ke_old  > 0.1 && skip > 1) skip /= 2;
  #endif
     ke_old = ke_sum1;
     ke_sum1 = 0.0;
     vz_sum = 0.0;
     for (ii=0; ii<np[0]; ii++)
       {                     
        energy = v_r[0][ii]*v_r[0][ii] + v_theta[0][ii]*v_theta[0][ii] + v_z[0][ii]*v_z[0][ii];
        ke_sum1 += energy;
        vz_sum += v_z[0][ii];
     }
#ifdef MPI_1D
     MPI_Allreduce(&vz_sum, &vz_sum_tot, 1, MPI_FLOAT, MPI_SUM, MPI_COMM_WORLD);
     I_z = Iz_norm*vz_sum_tot;
#else
     I_z =  Iz_norm*vz_sum;
#endif
  }
  if (I_z) E_z = source()/I_z;
/*#ifdef MPI_1D
  if (my_rank == 0) printf("time=%e, E_z=%lf, Iz=%lf, Ne=%d\n", atime, E_z/(ptopn*dr), I_z, np[0]);
#else
   printf("time=%e, E_z=%lf, Iz=%lf, Ne=%d\n", atime, E_z/(ptopn*dr), I_z, np[0]);
#endif*/
  yp[0] = rhon*rho[0]*vol[0];
  for (ii=1; ii<nc; ii++) yp[ii]=xp[ii]*(rhon*rho[ii] + a_phi[ii]*yp[ii-1]);

  phi[nc] = 0.0;
  for (ii=nc-1; ii>=0; ii--) phi[ii] = yp[ii] + c_phi[ii]*xp[ii]*phi[ii+1];
}
