/**********************************************************************
**   start
**   Read in input file, allocate arrays, normalise variables,
**   restore data from a previous run or load in a new particle
**   distribution and various other "setting up" steps
**   Parameter grid determines type of grid to be loaded:
**   grid = 0 non-uniform (constant volume) mesh
**   grid = 1 uniform mesh
**   grid = 2 linearly decreasing mesh
**   grid = 3 uniform mesh with 2 grid spacings
***********************************************************************/
#include "pdc1.h"
#include <xgrafix.h>

FILE *InputDeck;

void species(int initnp[NSMAX][2], int isp);
void Restore(char *filename);
int grid, nc0, profile;
SCALAR iinitn, dr1;
void unif_load(int initnp[NSMAX][2]);
void cos_load(int initnp[NSMAX][2], SCALAR iinitn);
void bes_load(int initnp[NSMAX][2], SCALAR iinitn);
void input_for_rad();

/*------------------------------------------------------------*/
void start()
{
  SCALAR Bz, f0;
  SCALAR ntemp, sum_nc, grad;
  char aachar[100];
  int i, j, isp, nc1, initnp[NSMAX][2];
  
/*---------------------------------------------*/
/* Open files and read in "general" parameters */
  
  if (!WasInputFileGiven) 
  {
  #ifdef MPI_1D
    if(my_rank == ROOT){
      puts("Syntax:");
      puts("\tPDC1 -i <inputdeck> -d [dumpfile]");
    }
  #else
    puts("Syntax:");
    puts("\tPDC1 -i <inputdeck> -d [dumpfile]");
  #endif
    exit(1);
  }
  InputDeck = fopen(theInputFile, "r");

  /*---------------------------------------------*/
  /* Read lines until we get to numbers */
  while (fscanf(InputDeck,"%d %d %d %lf %lf %lf %lf %lf %lf %lf",
		&nsp, &nc, &grid, &nc2p, &dt, &r0, &r1, &height, &epsilon, &Bz) <10)
    fscanf(InputDeck, "%s", aachar);

  
  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf %lf %lf",
		&rhoback, &backgrnd_i, &dde, &extr, &extl, &extc, &q_0) < 7)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%d %c %lf %lf %lf %lf %lf %lf", &ramped,
		&src, &dcbias, &dc_ramp, &acbias, &ac_ramp, &f0, &theta0) <8) 
    fscanf(InputDeck, "%s", aachar);

 if((src == 'W') ||(src == 'K'))
    {
      while (fscanf(InputDeck, "%d %lf %lf %lf %lf %lf %lf", &ramped2,
                &dcbias2, &dc_ramp2, &acbias2, &ac_ramp2, &f02, &theta02) <7) 
        fscanf(InputDeck, "%s", aachar);
    } 

  
  while (fscanf(InputDeck, "%d %d %d %d %d %d %d", &secondary, &ecollisional,
		&icollisional, &reflux, &nfft, &nsmoothing, &RT_flag) <7)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %d %lf %lf %d", &seec[0], &seec[1],
		&ionspecies, &pressure, &T_gas, &gas) <6)
    fscanf(InputDeck, "%s", aachar);
  
  /*---------------------------------------------*/
  /* Check for errors in the "general" input parameters */
  
  #ifdef MPI_1D
  if (RT_flag) {
    printf("For parallel version, RT simulation is not possible\n");
    exit(1);
  }
  #endif
  if (extl < 0. || extr < 0. || extc < 0.)
  {
  #ifdef MPI_1D
    if(my_rank == ROOT)
      printf("START-ERROR- external circuit improperly defined R,L,C < 0\n");
  #else
    printf("START-ERROR- external circuit improperly defined R,L,C < 0\n");
  #endif
    exit(1);
  }
  if (!dt)
  {
  #ifdef MPI_1D
    if(my_rank == ROOT)
      printf("START: dt == 0!!\n");
  #else
    printf("START: dt == 0!!\n");
  #endif
    exit(1);
  }
  if (nsp > NSMAX)
  {
  #ifdef MPI_1D
    if(my_rank == ROOT)
      printf("START: nsp > NSMAX.\n");
  #else
    printf("START: nsp > NSMAX.\n");
  #endif
    exit(1);
  }
    
  /* Check to see if nfft is an integer power of 2 */
  if(nfft)
  {
    for(i=0; i<= 31; i++)
    {
      if(nfft == (1<<i)) 
      {
	i=0;
	break;
      }
    }
    if(i)
    {
    #ifdef MPI_1D
      if(my_rank == ROOT)
        printf("START: nfft is not an integer power of 2.\n");
    #else
      printf("START: nfft is not an integer power of 2.\n");
    #endif
      exit(1);
    }
  }

  ng= nc+1;
  
  /*-----------------------------------*/
  /*  Set collision parameters         */
   mccptr =argonmcc;         /* gas uses argonmcc (without action) if collisions are off */
   if (ecollisional || icollisional)
   {
      switch (gas)
      {
         case ARGON:
	   mccptr =argonmcc; 
	   sigmaMptr =asigma1a; 
	   sigmaTptr =asigmaT; 
	   mueFacPtr = mue_factor; 
	   eion = 15.76;
	   eexc = 11.62;
	   e_me = 11.55;
	   e_miz = 4.16;
        #ifdef MPI_1D
	   if(my_rank == ROOT)
	     printf("Using anisotropic HBS cross-section set\n");
	#else
           printf("Using anisotropic HBS cross-section set\n");
        #endif
	   break;

         case ARGON_LK:
	   mccptr =argonmccLK; 
	   sigmaMptr =LK_asigma1a; 
	   sigmaTptr =LK_asigmaT; 
	   mueFacPtr = mue_factorLK; 
	   eion = 15.76;
	   eexc = 11.62;
	   e_me = 11.55;
	   e_miz = 4.16;
        #ifdef MPI_1D
	   if(my_rank == ROOT)
	     printf("Using isotropic L&K cross-section set\n");
	#else
           printf("Using isotropic L&K cross-section set\n");
        #endif
	   break;
      }
   }  


  /*---------------------------------------------*/
  /* Calculate normalised cell radii and volumes */

  r_grid = (SCALAR  *)malloc(ng*sizeof(SCALAR));
  dr_n = (SCALAR  *)malloc(ng*sizeof(SCALAR));

/* Constant cell volume mesh */
  if (grid == 0)
  {
      j_pointer = const_vol_grid;

      dr = -r0 + sqrt(r0*r0 + (r1*r1 - r0*r0)/nc);
      r0 /= dr;
      r1 /= dr;

      r_grid[0] = r0;
      r_grid[1] = r0+1;
      for (i=2; i<ng; i++) r_grid[i] = sqrt(2.0*r_grid[i-1]*r_grid[i-1] - r_grid[i-2]*r_grid[i-2]);
      for (i=0; i<nc; i++) dr_n[i] = r_grid[i+1] - r_grid[i];
  }
/* Constant uniform grid */
  else if(grid == 1)
  {
      j_pointer = unif_grid;
      dr = (r1-r0)/nc;
      r0 /=dr;
      r1 /=dr;

      for (i=0; i<ng; i++) r_grid[i] = r0 + i;
      for (i=0; i<ng; i++) dr_n[i] = 1;
  }
/* Linear dr mesh */
  else if(grid == 2)
  {
      j_pointer = linear_grid;

/* determine first dr */
      dr = 1.5*(r1 - r0)/nc; /* choose the first grid cell spacing - between 1.2 and 2.0 works best */
      r0 /= dr;
      r1 /= dr;

      sum_nc = 0.0;
      for (i=1; i<nc; i++) sum_nc += i;
      grad = (r1 - r0 - nc)/sum_nc;
      for (i=0; i< nc; i++) dr_n[i] = grad*i + 1.0;
      r_grid[0] = r0;
      for (i=1; i< ng; i++) r_grid[i] = r_grid[i-1] + dr_n[i-1];
  }
/* Uniform mesh with two spacings */
  else if(grid == 3)
  {
     j_pointer = split_grid;

     nc0 = 0.2*nc;
     nc1 = nc - nc0;
     dr = 0.5*(r1 - r0)/nc0;                      /* 1/4 length with nc0 cells */
     dr1 = 0.5*(r1 - r0)/nc1;                     /* 3/4 length with nc1 cells */
     r0 /= dr;
     r1 /= dr;

     for (i=0; i< nc0; i++) dr_n[i] = 1.0;         /* normalise by dr */
     for (i=nc0; i< nc; i++) dr_n[i] = dr1/dr;
     r_grid[0] = r0;
     for (i=1; i< ng; i++) r_grid[i] = r_grid[i-1] + dr_n[i-1];
  }

  vol = (SCALAR  *)malloc(ng*sizeof(SCALAR));
  vol[0] = r0 + 1.0/3.0; 
  vol[nc] = dr_n[nc-1]*(r_grid[nc] - dr_n[nc-1]/3.0); 
  for ( i=1; i< nc; i++) 
    vol[i] = r_grid[i]*(dr_n[i] + dr_n[i-1]) + (dr_n[i]*dr_n[i] - dr_n[i-1]*dr_n[i-1])/3.0;

  /*------------------------------------*/
  /* Conversion to dimensionless units */
  
  gas_den = NperTORR*pressure/(T_gas +DBL_MIN);
  area0 = TWOPI*r0*dr*height;
  theta0 *= TWOPI/360;
  Trf =1/(f0+DBL_MIN);
  cycle = Trf/dt;
  w0 = TWOPI*f0;
  w02 = TWOPI*f02;
  epsilon*= EPS0;
  q_1 = q_2 = q_3 = q_0;
  reactor_vol = PI*height*dr*dr*(r1*r1 - r0*r0);  /* in real units */
  rescale_on=1;
  
  /*-----------------------------------------------*/
  /* Calculate normalisation parameters */
  me_e = EMASS/ECHARGE;
  vscale = dr/dt;
  ptopn = (ECHARGE/EMASS)*dt*dt/(dr*dr);  /* Normalisation for potential */
  dntod =  nc2p/(PI*height*dr*dr); /* Normalisation for density */

  /* Check ramped source parameters                      */
  if(ramped)
  {
    if(dc_ramp*dcbias < 0.0)
    {
     #ifdef MPI_1D
       if(my_rank == ROOT){
         fprintf(stderr, "\nWARNING: Ramp and dcbias have opposite signs.");
         fprintf(stderr, "\nThe external source will be discontinuous!\n\n");
       }
     #else
       fprintf(stderr, "\nWARNING: Ramp and dcbias have opposite signs.");
       fprintf(stderr, "\nThe external source will be discontinuous!\n\n");
     #endif
    }
    if (dc_ramp) risetime= fabs(dcbias)/fabs(dc_ramp);
  }
 if(ramped2)
  {
    if(dc_ramp2*dcbias2 < 0.0)
    {
     #ifdef MPI_1D
       if(my_rank == ROOT){
         fprintf(stderr, "\nWARNING: Ramp and dcbias have opposite signs.");
         fprintf(stderr, "\nThe external source will be discontinuous!\n\n");
       }
     #else
       fprintf(stderr, "\nWARNING: Ramp and dcbias have opposite signs.");
       fprintf(stderr, "\nThe external source will be discontinuous!\n\n");
     #endif
    }
    if (dc_ramp2) risetime2= fabs(dcbias2)/fabs(dc_ramp2);    
  }


  /*-----------------------------------*/
  /* Allocate space for field parameters */
  
  r_array= (SCALAR  *)malloc(ng*sizeof(SCALAR));
  for(i=0; i< ng; i++) r_array[i]= r_grid[i]*dr;

  sp_n= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
  sp_n_k= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
  sp_n_0= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
  sp_n_mcc= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
  for(i=0; i<nsp; i++)	
  {
     sp_n[i]= (SCALAR  *)malloc(ng*sizeof(SCALAR));
     sp_n_k[i]= (SCALAR  *)malloc(ng*sizeof(SCALAR));
     sp_n_0[i]= (SCALAR  *)malloc(ng*sizeof(SCALAR));
     sp_n_mcc[i]= (SCALAR  *)malloc(ng*sizeof(SCALAR));
      
     for(j=0; j<ng; j++)
       sp_n[i][j] = sp_n_0[i][j] = sp_n_k[i][j] = sp_n_mcc[i][j] = 0.0;
  }
  
  rho= (SCALAR  *)malloc(ng*sizeof(SCALAR));
  rho_unfilt= (SCALAR  *)malloc(ng*sizeof(SCALAR));
  phi= (SCALAR  *)malloc(ng*sizeof(SCALAR));
  efield  = (SCALAR  *)malloc(ng*sizeof(SCALAR));
  phi2 = (SCALAR  *)malloc(ng*sizeof(SCALAR));
  efield2  = (SCALAR  *)malloc(ng*sizeof(SCALAR));

  for (i=0; i<ng; i++) rho_unfilt[i] = 0.0;
  
  /**** Allocate Dianostic arrays  *******/
  if(theRunWithXFlag)
  {
    jdote = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jzdotez = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_t = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jr_unfilt = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jr_prof = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jt_prof = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_z = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jz_prof = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    mu_prof = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ke_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ker_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ket_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    kez_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ke_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ker_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ket_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    kez_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_r_avg =  (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_z_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    j_t_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    u_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    u_t = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    u_z = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Ttotal = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    //    Ttotal2 = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tr = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tt = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tz = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    T_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tr_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tt_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Tz_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Dr_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drr_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drt_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drz_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Dr_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drr_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drt_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Drz_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    
    ke_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    T_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    Dr_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));


    for(isp=0; isp<nsp; isp++) {
       T_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ke_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Dr_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       //       jdote_sum = (SCALAR **)malloc(ng*sizeof(SCALAR *));
       //       jzdotez_sum = (SCALAR **)malloc(ng*sizeof(SCALAR *));
    }

   
 
    for(isp=0; isp<nsp; isp++) 
    {    
       j_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jr_unfilt[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jr_prof[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       j_t[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jt_prof[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       j_z[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jz_prof[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       mu_prof[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ke_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ker_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ket_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       kez_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ke_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ker_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ket_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       kez_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ker_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       ket_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       kez_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       j_r_avg[isp] =  (SCALAR *)malloc(ng*sizeof(SCALAR));
       j_z_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       j_t_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       u_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       u_t[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       u_z[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Ttotal[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       //       Ttotal2[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tr[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tt[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tz[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Dr_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drr_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drt_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drz_r[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       T_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tr_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tt_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Tz_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Dr_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drr_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drt_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       Drz_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jdote[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jzdotez[isp] = (SCALAR *) malloc(ng*sizeof(SCALAR));
       for(j=0; j<ng; j++) jr_unfilt[isp][j] = jdote[isp][j] = jzdotez[isp][j]
          = jr_prof[isp][j] = jz_prof[isp][j] =jt_prof[isp][j] = mu_prof[isp][j]
	  = ke_avg[isp][j] = ker_avg[isp][j] = ket_avg[isp][j] = kez_avg[isp][j]
	  = ke_r[isp][j] = ker_r[isp][j] = ket_r[isp][j] = kez_r[isp][j]
	  = Ttotal[isp][j] = Tr[isp][j] = Tt[isp][j] = Tz[isp][j] 
	  = T_avg[isp][j]=Tr_avg[isp][j]= Tt_avg[isp][j]= Tz_avg[isp][j]
          =Dr_avg[isp][j]=Drr_avg[isp][j]= Drz_avg[isp][j]=Drt_avg[isp][j] 
	  = Dr_r[isp][j] = Drr_r[isp][j] = Drt_r[isp][j] = Drz_r[isp][j] = 0.0;
    }

  /* Allocate space for time history diagnostics */
  
    t_array= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    ke_tot= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    te_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    ese_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    sigma_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    Ez_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    mu_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    mu_hist2 = (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    mu_hist3 = (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    Iz_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    Iz_hist2= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    nu_iz_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    nu_ex_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    nu_el_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
  
    com_cur_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    LHSphi_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    midphi_hist= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    Isheathphi_hist = (SCALAR *) malloc(HISTMAX*sizeof(SCALAR));
    Osheathphi_hist = (SCALAR *) malloc(HISTMAX*sizeof(SCALAR));
    IOphi_ratio_hist = (SCALAR *) malloc(HISTMAX*sizeof(SCALAR));
    Isheath=(SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    Osheath= (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    plasmawidth = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
  
    com_power_hist = (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    avg_pow_hist = (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
  
    np_hist= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
    np_hist2= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
    kes_hist= (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));
    Ts_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    us_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    usr_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ust_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    usz_hist = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    flux = (SCALAR  **)malloc(nsp*sizeof(SCALAR  *));

    C_hist = (SCALAR **)malloc(3*sizeof(SCALAR *));		/* LEEHJ */
    C_hist[0] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
    C_hist[1] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));

    for(i=0; i<nsp; i++)
    {
      np_hist[i]= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
      np_hist2[i]= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
      kes_hist[i]= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
      Ts_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      us_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      usr_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      ust_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      usz_hist[i] = (SCALAR *)malloc(HISTMAX*sizeof(SCALAR));
      flux[i]= (SCALAR  *)malloc(HISTMAX*sizeof(SCALAR));
    }
  /*--------------------------------------------------*/
  /* Allocate space for frequency history diagnostics and
  time-average diagnositics */
  if(nfft)
    {

/**** Profile and time-averaged profiles of diagnostic arrays ****/
      density= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      den_sum= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      den_ave= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      den2_avg= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
      den2_sum= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jdote_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jzdotez_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jdote_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jzdotez_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jr_prof_sum = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    jr_prof_avg = (SCALAR **)malloc(nsp*sizeof(SCALAR *));


      for(isp=0; isp<nsp; isp++)
      {
         density[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
         den_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
         den_ave[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
         den2_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
         den2_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
	 
       jdote_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jzdotez_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jdote_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jzdotez_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jr_prof_sum[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
       jr_prof_avg[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));

         for(i=0; i< ng; i++){
	   den_sum[isp][i]= den_ave[isp][i]= 0.0;
	   den2_sum[isp][i] = den2_avg[isp][i]=0.0;
	   jdote_sum[isp][i] = jdote_avg[isp][i]=0.0;
	   jzdotez_sum[isp][i] = jzdotez_avg[isp][i] = 0.0;
	 }
      }
      phi_sum= (SCALAR *)malloc(ng*sizeof(SCALAR));
      phi_ave= (SCALAR *)malloc(ng*sizeof(SCALAR));
      for(i=0; i< ng; i++) phi_sum[i]= phi_ave[i]= 0.0;

      e_sum= (SCALAR *)malloc(ng*sizeof(SCALAR));
      e_ave= (SCALAR *)malloc(ng*sizeof(SCALAR));
      for(i=0; i< ng; i++) e_ave[i]= e_sum[i]= 0.0;

      rho_ave= (SCALAR *)malloc(ng*sizeof(SCALAR));
      rho_sum= (SCALAR *)malloc(ng*sizeof(SCALAR));
      for(i=0; i< ng; i++) rho_sum[i]= 0.0;

/**** Frequency history arrays ****/
      freq_hi= nfft/2;
      df= 1/(nfft*dt);

      f_array= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      for(i=0; i< nfft; i++) f_array[i]= i*df;
      cur_hist= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      phi_hist= (SCALAR  **)malloc(2*sizeof(SCALAR  *));
      phi_hist[0]= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      phi_hist[1]= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      pow_hist= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
    
      Local_t_array = (SCALAR  *)malloc(nfft*sizeof(SCALAR));
    
/**** FFT arrays ****/
      cur_fft= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      phi_fft= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      mphi_fft=(SCALAR  *)malloc(nfft*sizeof(SCALAR));
      pow_fft= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
      z_fft= (SCALAR  *)malloc(nfft*sizeof(SCALAR));
    }
  } /*end of if (theRunWithXFlag) loop */
  
  /*------------------------------------------*/
  /* Set up each "species" parameters         */
  vrscale=0.0;
  for (isp=0; isp<nsp; ++isp)
  {
    species(initnp,isp);
    
    /* Variables for converting from physical to code units */
    q_ratio[isp] = charge[isp]/ECHARGE;
    qm[isp]= charge[isp]/mass[isp];
    wc[isp] = qm[isp]*Bz*dt;
    Escale[isp] = 0.5*vscale*vscale*mass[isp]/ECHARGE;
    intoi[isp] = charge[isp]*nc2p/dt;      /* Normalisation for current */
    a_scale[isp] = qm[isp]*me_e*sp_k[isp];

    /* Renormalize all velocities */
    v0[isp][0] /= vscale;
    v0[isp][1] /= vscale;
    vc[isp][0] /= vscale;
    vc[isp][1] /= vscale;
    vt[isp][0] /= vscale;
    vt[isp][1] /= vscale;
    v0_th[isp] /= vscale;
    vt_th[isp] /= vscale;
    v0_z[isp]  /= vscale;
    vt_z[isp]  /= vscale;
    
    emin[isp] /= Escale[isp];
    de[isp]   /= Escale[isp];
    de_mid[isp]   /= Escale[isp];
    emin_mid[isp] /= Escale[isp];
   
    rs_mid[isp] /= dr;
    rf_mid[isp] /= dr;
  }
  
  if (RT_flag) input_for_rad();                          /* LEEHJ ; RAD-TR */
  else nc_RT=nc;
  ng_array=(SCALAR *)malloc(nc_RT*sizeof(SCALAR));
  for (j=0;j<nc_RT;j++) ng_array[j]=1.0;

  if (nc%nc_RT==0) nc_ratio=nc/nc_RT;
  else {
    puts("nc should be a multiple of nc_RT");
    exit(-1);
  }
  fclose(InputDeck);
  if (RT_flag) start_rad();

/* Normalise parameters which depend on nc2p (can change in species
sub-routine */
  rhoback /= dntod*ECHARGE;
  rhon = ptopn*(ECHARGE/EPS0)*nc2p/(PI*height);

  if(!vrscale) vrscale = 1e4;
  vrscale *= 5;

  /* Allocate particle arrays */
  
  r  = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  v_r = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  v_theta= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  v_z = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  angular_mtm = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  
  for(isp=0; isp< nsp; isp++)
  {
    r[isp]  = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
    v_r[isp] = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
    v_theta[isp]= (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
    v_z[isp] = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
    angular_mtm[isp] = (SCALAR *)malloc(maxnp[isp]*sizeof(SCALAR));
    for(j=0; j<maxnp[isp]; j++)
      r[isp][j]=v_r[isp][j]=v_theta[isp][j]=v_z[isp][j]=angular_mtm[isp][j]=0.0;
  }
  
  if (nsp && (!r[nsp-1] || !v_r[nsp-1] || !v_theta[nsp-1] || !v_z[nsp-1]))
  {
  #ifdef MPI_1D
    if(my_rank == ROOT)
      puts("START: r, v malloc failed");
  #else
    puts("START: r, v malloc failed");
  #endif
    exit(1);
  }
   
  /*************************************************/
  /* Allocate space for the dist. functions arrays */
  
  if (theRunWithXFlag)
  {

/* Allocate arrays for velocity distribution diagnostic */
     if (vel_dist_accum)
     {
       vr_dist = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
       vth_dist = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
       vz_dist = (SCALAR ***)malloc(nsp*sizeof(SCALAR **));
       for(isp=0; isp<nsp; isp++)
       {
          vr_dist[isp] = (SCALAR **)malloc(ng*sizeof(SCALAR*));
          vth_dist[isp] = (SCALAR **)malloc(ng*sizeof(SCALAR*));
          vz_dist[isp] = (SCALAR **)malloc(ng*sizeof(SCALAR*));
          for (i=0; i<ng; i++)
          {
             vr_dist[isp][i] = (SCALAR *)calloc(nvrbin[isp],sizeof(SCALAR));
             vth_dist[isp][i] = (SCALAR *)calloc(nvtbin[isp],sizeof(SCALAR));
             vz_dist[isp][i] = (SCALAR *)calloc(nvzbin[isp],sizeof(SCALAR));
          }
       }
       vr_array = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
       vth_array = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
       vz_array = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
       for(isp=0; isp<nsp; isp++)
       {
          vr_array[isp] = (SCALAR *)malloc(nvrbin[isp]*sizeof(SCALAR));
          vth_array[isp] = (SCALAR *)malloc(nvtbin[isp]*sizeof(SCALAR));
          vz_array[isp] = (SCALAR *)malloc(nvzbin[isp]*sizeof(SCALAR));
       }
     }

/* energy distribution functions */
    fe      = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    e_array = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    ftheta  = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    th_array= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    fe_mid = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    fe_mid_show = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
    emid_array= (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  
    for(isp=0; isp<nsp; isp++)
    {
      fe[isp]     = (SCALAR *)malloc(nbin[isp]*sizeof(SCALAR));
      ftheta[isp]  = (SCALAR *)malloc(nbin[isp]*sizeof(SCALAR));
      e_array[isp]= (SCALAR *)malloc(nbin[isp]*sizeof(SCALAR));
      th_array[isp]= (SCALAR *)malloc(nbin[isp]*sizeof(SCALAR));
      fe_mid[isp]  = (SCALAR *)malloc(nbin_mid[isp]*sizeof(SCALAR));
      fe_mid_show[isp]  = (SCALAR *)malloc(nbin_mid[isp]*sizeof(SCALAR));
      emid_array[isp]= (SCALAR *)malloc(nbin_mid[isp]*sizeof(SCALAR));
    }
  
    for(isp=0; isp<nsp; isp++)
    {
      for(i=0; i<nbin[isp]; i++)
      {
        fe[isp][i]      = 0.0;
        ftheta[isp][i]  = 0.0;
        e_array[isp][i] = emin[isp] +i*de[isp];
        th_array[isp][i]= i*dtheta[isp];
      }
      for (i=0; i<nbin_mid[isp]; i++)
      {
          emid_array[isp][i]= emin_mid[isp] +(i+.5)*de_mid[isp];
          fe_mid[isp][i] = fe_mid_show[isp][i] = 0.0;
      }
    }   
  }

#ifdef MPI_1D

  /* MPI parameters */
  counts = (int *)malloc(num_procs*sizeof(int));
  displs = (int *)malloc(num_procs*sizeof(int));

  /* MPI total arrays */
  sp_n_tot = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  for(isp=0; isp<nsp; isp++) {
      iwall_tot[isp] = 0;
      sp_n_tot[isp] = (SCALAR *)malloc(ng*sizeof(SCALAR));
      for(j=0; j<ng; j++)
         sp_n_tot[isp][j] = 0.0;
  }

  r_tot  = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vr_tot = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vtheta_tot = (SCALAR **)malloc(nsp*sizeof(SCALAR *));
  vz_tot = (SCALAR **)malloc(nsp*sizeof(SCALAR *));

  for(isp=0; isp<nsp; isp++) {
    r_tot[isp]  = (SCALAR *)malloc(num_procs*maxnp[isp]*sizeof(SCALAR));
    vr_tot[isp] = (SCALAR *)malloc(num_procs*maxnp[isp]*sizeof(SCALAR));
    vtheta_tot[isp] = (SCALAR *)malloc(num_procs*maxnp[isp]*sizeof(SCALAR));
    vz_tot[isp] = (SCALAR *)malloc(num_procs*maxnp[isp]*sizeof(SCALAR));

    for(j=0; j<maxnp[isp]; j++)
      r_tot[isp][j]=vr_tot[isp][j]=vtheta_tot[isp][j]=vz_tot[isp][j]=0.0;

  }  

  if (nsp && (!r_tot[nsp-1] || !vr_tot[nsp-1] || !vtheta_tot[nsp-1] ||
              !vz_tot[nsp-1])) {
    if(my_rank == ROOT)
      puts("START: r, v malloc failed");
    exit(-1);
  } 

#endif /* MPI_1D */
  

  /*********************************************************/
  /* Load in all species' particles, properly distributed. */
  /* Start from a dump file if given otherwise load the    */
  /* system with the initial distribution                  */

  if (WasDumpFileGiven) 
  {
       ntemp = nc2p;
       Restore(theDumpFile);

 /* Rescale normalisation variables using nc2p from the Dump file */
       for (isp=0; isp<nsp; isp++) intoi[isp] *= nc2p/ntemp; 
       rhon *= nc2p/ntemp; 
  }
  else if (profile == 1)
       unif_load(initnp);
  else if (profile == 2)
     bes_load(initnp,iinitn);
  else if (profile == 3)
     cos_load(initnp,iinitn);


}	/* End START */

/***************************************************************/
/*   If species parameters exist read them from input file    
**   Shape of density profile determined by parameter profile:
**   profile = 1   uniform profile
**   profile = 2   bessel function profile
**   profile = 3   cosine profile
*/

void species(int initnp[NSMAX][2], int isp)
{
  char aachar[200];
  int np_start, ii;
  SCALAR j0_l, j0_r, v0r_o, v0r_i, vcr_o, vcr_i, vtr_o, vtr_i,
        v0_t, vt_t, v0z, vtz;
  SCALAR c0, temin, temax, temin_mid, temax_mid, area1;
  
  /* Read in SPECIES parameters... */
  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf %d %d", &charge[isp], &mass[isp],
		&j0_l, &j0_r, &iinitn, &profile, &sp_k[isp]) <7)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %lf %lf %lf %lf ", &v0r_o, &v0r_i, &vtr_o,
		&vtr_i, &vcr_o, &vcr_i) <6)
    fscanf(InputDeck, "%s", aachar);
  
  while (fscanf(InputDeck, "%lf %lf %lf %lf", &v0_t, &vt_t, &v0z, &vtz) <4)
    fscanf(InputDeck, "%s", aachar);

/* Energy distribution at outside wall */
  while (fscanf(InputDeck, "%d %lf %lf %d", &nbin[isp], &temin,
		&temax, &maxnp[isp]) <4)
    fscanf(InputDeck, "%s", aachar);

/* Energy distribution at plasma centre */
  while (fscanf(InputDeck, "%d %lf %lf %lf %lf", &nbin_mid[isp], &temin_mid,
                                           &temax_mid, &rs_mid[isp], &rf_mid[isp]) <5)
     fscanf(InputDeck, "%s", aachar);

/* Velocity distribution at plasma centre */
  while (fscanf(InputDeck, "%lf %lf %d %lf %lf %d %lf %lf %d", &vr_l[isp], &vr_u[isp], &nvrbin[isp],
            &vt_l[isp], &vt_u[isp], &nvtbin[isp], &vz_l[isp], &vz_u[isp], &nvzbin[isp]) <9)
     fscanf(InputDeck, "%s", aachar);

  if (nvrbin[isp]||nvtbin[isp]||nvzbin[isp]) vel_dist_accum=1;
  
  /* Assign input parameters to arguments */
    v0[isp][0] = fabs(v0r_o);          /* drift velocity v > 0 (moving out) */
    v0[isp][1] = -fabs(v0r_i);         /* drift velocity v < 0 (moving in) */
    vc[isp][0] = fabs(vcr_o);          /* cutoff velocity for v > 0, thermal dist */
    vc[isp][1] = -fabs(vcr_i);         /* cutoff velocity for v < 0, thermal dist */
    vt[isp][0] = fabs(vtr_o);          /* thermal velocity, v > 0 */
    vt[isp][1] = -fabs(vtr_i);         /* thermal velocity, v < 0 */
    v0_th[isp] = v0_t;                 /* drift velocity in theta direction */
    vt_th[isp] = fabs(vt_t);           /* thermal velocity in theta direction */
    v0_z[isp]  = v0z;                  /* drift velocity in z direction */
    vt_z[isp]  = fabs(vtz);            /* thermal velocity in z direction */
    //    v0[isp][0] = v0r_o;          /* drift velocity v > 0 (moving out) */
    //    v0[isp][1] = -v0r_i;         /* drift velocity v < 0 (moving in) */
    //    vc[isp][0] = vcr_o;          /* cutoff velocity for v > 0, thermal dist */
    //    vc[isp][1] = -vcr_i;         /* cutoff velocity for v < 0, thermal dist */
    //    vt[isp][0] = vtr_o;          /* thermal velocity, v > 0 */
    //    vt[isp][1] = -vtr_i;         /* thermal velocity, v < 0 */
    //    v0_th[isp] = v0_t;                 /* drift velocity in theta direction */
    //    vt_th[isp] = vt_t;           /* thermal velocity in theta direction */
    //    v0_z[isp]  = v0z;                  /* drift velocity in z direction */
    //    vt_z[isp]  = vtz;            /* thermal velocity in z direction */

/* parameters for energy and velocity distribution diagnostics */
  emin[isp]  = temin;              /* min energy measured in energy dist diag */ 
  de[isp]	  = (temax-temin)/(nbin[isp]-1);      /* energy bin size */
  dtheta[isp]= 0.5*PI/(nbin[isp]-1);                    /* angular bin size */
  emin_mid[isp] = temin_mid;
  de_mid[isp] = (temax_mid-temin_mid)/(nbin_mid[isp]);
  
  /*determine max vel of right & left moving particles for making distribution function */
  vrscale= max(vrscale, v0[isp][0]+vt[isp][0]);
  vrscale= max(vrscale, -v0[isp][1]+vt[isp][1]);

  /* Check that nc2p and iinitn produce # sim particles w/in range */
  if (!WasDumpFileGiven) 
  {
     if (profile == 1) 
     {
       np_start = iinitn*reactor_vol/nc2p + 0.5;
      #ifdef MPI_1D
         if (my_rank == ROOT) 
         {   
           printf(" den_init = %e vol = %lf \n",iinitn, reactor_vol);
           printf("np = %d maxnp = %d ",np_start,maxnp[isp]);
           printf("nc2p = %e \n",nc2p);
         }
      #else
         printf(" den_init = %e vol = %lf \n",iinitn, reactor_vol);
         printf("np = %d maxnp = %d ",np_start,maxnp[isp]);
         printf("nc2p = %e \n",nc2p);
      #endif
     }
     else 
     {
       np_start = 0;
       c0 = iinitn/dntod;
       if (profile == 2)
       {
         for (ii=0; ii<=nc-2; ii++)
           np_start += c0*J_0((r_grid[ii]-r0)/(r1-r0))*(r_grid[ii+1]*r_grid[ii+1] - r_grid[ii]*r_grid[ii]);
       }
       else if (profile == 3)
       {
         for (ii=0; ii<=nc-2; ii++)
           np_start += c0*cos(PI*0.5*(SCALAR)(r_grid[ii]-r0)/(r1-r0))*(r_grid[ii+1]*r_grid[ii+1] - r_grid[ii]*r_grid[ii]);
       }
      #ifdef MPI_1D
        if (my_rank == ROOT) 
        {
          printf(" n_pk = %e \n",iinitn);
          printf("np = %d maxnp = %d ",np_start,maxnp[isp]);
          printf("nc2p = %e \n",nc2p);
	}
      #else
        printf(" n_pk = %e \n",iinitn);
        printf("np = %d maxnp = %d ",np_start,maxnp[isp]);
        printf("nc2p = %e \n",nc2p);
      #endif
     }   /* end of if (profile) loop */

  /* increase nc2p & reduce # simulation particles */
     if (np_start > maxnp[isp])                  
     {
         nc2p *= np_start/(0.9*maxnp[isp]);
         np_start = 0.9*maxnp[isp];
       #ifdef MPI_1D
         if (my_rank == ROOT)
           printf("SPECIES - too many simulation particles, decreasing to %d and increasing nc2p to %e  \n\n",np_start, nc2p);
       #else
         printf("SPECIES - too many simulation particles, decreasing to %d and increasing nc2p to %e  \n\n",np_start, nc2p);  
       #endif
     }
  /* decrease nc2p & increase # simulation particles*/
     if (iinitn && np_start < min_np)
     { 
         nc2p *= np_start/(1.2*min_np);
         np_start = 1.2*min_np;
      #ifdef MPI_1D
        if (my_rank == ROOT) 
          printf("SPECIES - too few simulation particles, increasing to %d and decreasing nc2p to %e \n\n",np_start, nc2p);
      #else
        printf("SPECIES - too few simulation particles, increasing to %d and decreasing nc2p to %e \n\n",np_start, nc2p);
      #endif
     }
 /* If particles are moving both left and right */
     if((vt[isp][0] || v0[isp][0]) && (vt[isp][1] || v0[isp][1])) 
     {
       initnp[isp][0] = 0.5*np_start;
       initnp[isp][1] = 0.5*np_start;
     }
/* If particles are moving only to the left */
     else if((vt[isp][1] || v0[isp][1]) && (!vt[isp][0]  && !v0[isp][0])) 
     {
       initnp[isp][0] = 0.0;
       initnp[isp][1] = np_start;
     }
/* If particles are moving only to the right, or are stationary */
     else 
     {
       initnp[isp][0] = np_start;
       initnp[isp][1] = 0.0;
     }

  }    /* end of loop for initialising density */

/* MPI: Divide particles among procs */
     #ifdef MPI_1D
       maxnp[isp] = floor(0.5 + ((SCALAR) maxnp[isp])/num_procs);
       iinitn = floor(0.5 + ((SCALAR)iinitn)/num_procs);
       initnp[isp][0] = floor(0.5 + ((SCALAR)initnp[isp][0])/num_procs);
       initnp[isp][1] = floor(0.5 + ((SCALAR)initnp[isp][1])/num_procs);
     #endif

  /* j0_l & j0_r = injected current density from left & right electrodes */
  /* Set enter = no. of simulation particles injected PER DT */
  area1 = TWOPI*r1*height;               /* area outer electrode */
  enter[isp][0] = 0.0;
  if (r0>0.0) enter[isp][0] = fabs(j0_l*area0*dt*sp_k[isp]/(charge[isp]*nc2p));
  enter[isp][1] = fabs(j0_r*area1*dt*sp_k[isp]/(charge[isp]*nc2p));

  #ifdef MPI_1D
    /* MPI: Divide injected particles among procs */
    enter[isp][0] /= num_procs;
    enter[isp][1] /= num_procs;
  #endif

  if (enter[isp][0]> 0.) inject[isp] = 1;
  if (enter[isp][1]> 0.) inject[isp] = 1;

}	/* End SPECIES */

/***************************************************************/
/* Bessel's function calc, for loading particles               */
SCALAR J_0(SCALAR z)
{
   SCALAR x, j0;

   z *= 2.4;
   x = 0.25*z*z;
   j0 = 1 - x + 0.25*x*x - x*x*x/36.0;
   return (j0);
}
/*------------------------- LEEHJ -----------------------------*/
void input_for_rad()
{
  char aachar[80];
  SCALAR k0_input;

  while (fscanf(InputDeck, "%d %lf %c %lf %lf %d", &nc_RT, &lambda0, &Lineshape,
         &A_ki, &k0_input, &sp_k_ex)<6)
    fscanf(InputDeck, "%s", aachar);

  while (fscanf(InputDeck, "%lf %lf %d", &xmin, &xmax, &xbin) <3)
    fscanf(InputDeck, "%s", aachar);

  if      (Lineshape=='L' || Lineshape=='l') { lineshape=1; Cx = 1/M_PI;       }
  else if (Lineshape=='D' || Lineshape=='d') { lineshape=2; Cx = 1/sqrt(M_PI); }
  else if (Lineshape=='V' || Lineshape=='v') { lineshape=3; Cx = 1/M_PI;       }
  else {
    puts("Select proper lineshpae\n");
    exit(1);
  }
  tau=1e-8/A_ki;
  A_ki*=1e8;

  if (lineshape==1) {
    delta_nu=1.0555e-7*1.532*0.2214*lambda0*1e-9*gas_den;
    k0=lambda0*lambda0*A_ki*Cx*3/(4*M_PI*delta_nu);
    k0*=1e-18*gas_den;       /* 1e-18 = 1e-9[m]*1e-9[m]         */
  }
  else if (lineshape==2) {
    delta_nu=2*sqrt(2*log(2)*T_gas*1.602/(mass[1]*1e19))/lambda0*1e9;
    k0=lambda0*lambda0*A_ki*Cx*3/(4*M_PI*delta_nu)*sqrt(log(2));
    k0*=1e-18*gas_den;       /* 1e-18 = 1e-9[m]*1e-9[m]         */
  }
  I_factor=2/pow(lambda0,3)*6.6261*2.99792458*10/gas_den/3.0;
  lambda0*=1e-9;        /* lambda0[nm] is normalized as nm */
  printf("gas_den=%lf,\t",gas_den);
  printf("delta_nu=%lf GHz\t k0=%e\n",delta_nu*1e-9,k0);
  k0=k0_input;
}
/***************************************************************/
/* Routines to calculate nearest mesh point to particle position
** used in field and move. Depends on the type of mesh used in the simulation
*/
/* ------------------------------------------------------------------- */
int unif_grid(int isp, int ii)
{
   int j;
   j = r[isp][ii] - r0;
   return(j);
}
/* ------------------------------------------------------------------- */
int const_vol_grid(int isp, int ii)
{
   int j;
   j = (r[isp][ii]*r[isp][ii] - r0*r0)/(2.0*r0 + 1.0);
   return(j);
}
/* ------------------------------------------------------------------- */
int linear_grid(int isp, int ii)
{
   int j;

/* Get approximate position of nearest grid to particle at r */
   j = r[isp][ii] - r0;

/* Increment j until nearest grid point found */
   while (r_grid[j+1] < r[isp][ii] && j < ng) j++;

   return(j);
}
/* ------------------------------------------------------------------- */
int split_grid(int isp, int ii)
{
   int j;
   if (r[isp][ii] <= r_grid[nc0]) j = r[isp][ii] - r0;
   else j = nc0 + (r[isp][ii] - r_grid[nc0])*dr/dr1;
   return(j);
}
/***************************************************************/
